/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QKeyEvent>
#include <QSettings>
#include <QHBoxLayout>
#include <QGroupBox>
#include <QLabel>
#include <QCheckBox>
#include <QSpinBox>
#include <QAction>
#include <QApplication>
#include <QMessageBox>

#include "global.h"
#include "lineedits.h"
#include "calculator.h"

//QCalculatorFormulaEdit

QCalculatorFormulaEdit::QCalculatorFormulaEdit(const QString &text, QWidget *parent)
    : QTextEdit(text, parent)
{
}

void QCalculatorFormulaEdit::keyPressEvent(QKeyEvent *event)
{
    switch (event->key())
    {
    case Qt::Key_Enter:
    case Qt::Key_Return:
        if (event->modifiers() & Qt::ControlModifier)
        {
            emit okSignal();
        }
        else
        {
            QTextEdit::keyPressEvent(event);
        }
        break;
    default:
        QTextEdit::keyPressEvent(event);
    }
}

// QCalculatorDialog

QCalculatorDialog::QCalculatorDialog(bool a_fromLineEdit, const QString &a_objectName, const QString &title, QWidget *parent)
    : QDialog(parent)
{
    setWindowIcon(QIcon(":images/calc.svg"));
    setWindowTitle(title);

    readSettings();
    QHBoxLayout *mainLayout = new QHBoxLayout(this);
    QVBoxLayout *verticalLayout = new QVBoxLayout;
    QVBoxLayout *svgLayout = new QVBoxLayout;

    QGroupBox *editareGroupBox = new QGroupBox(this);

    QGridLayout *editareLayout = new QGridLayout(editareGroupBox);

    QLabel *formulaLabel = new QLabel(editareGroupBox);
    formulaEdit = new QCalculatorFormulaEdit(formula, editareGroupBox);
    formulaLabel->setBuddy(formulaEdit);
    formulaLabel->setAlignment(Qt::AlignTop);

    QFont font = formulaEdit->font();
    font.setPointSize(font.pointSize() + editFontPlus);
    formulaEdit->setFont(font);

    QLabel *resultLabel = new QLabel(editareGroupBox);
    resultEdit = new QFairyLineEdit("",editareGroupBox);
    //insertLineEditToVerify(resultEdit);
    resultLabel->setBuddy(resultEdit);

    editareLayout->addWidget(formulaLabel,0,0);
    editareLayout->addWidget(formulaEdit,0,1);

    editareLayout->addWidget(resultLabel,1,0);
    editareLayout->addWidget(resultEdit,1,1);

    verticalLayout->addWidget(editareGroupBox);

    QGroupBox *optionsGroupBox = new QGroupBox(this);

    QHBoxLayout *optionsLayout = new QHBoxLayout(optionsGroupBox);

    thousandCheckBox = new QCheckBox(tr("Separator mii"), optionsGroupBox);
    thousandCheckBox->setChecked(thousandSeparator);

    QLabel *decimalLabel = new QLabel(optionsGroupBox);
    decimalSpinBox = new QSpinBox(optionsGroupBox);
    decimalSpinBox->setRange(0,7);
    decimalSpinBox->setValue(numberOfDecimals);
    decimalLabel->setBuddy(decimalSpinBox);

    optionsLayout->addWidget(decimalLabel);
    optionsLayout->addWidget(decimalSpinBox);
    optionsLayout->addWidget(thousandCheckBox);
    optionsLayout->addStretch(1);

    verticalLayout->addWidget(optionsGroupBox);

    QGroupBox *buttonsGroupBox = new QGroupBox(this);

    QGridLayout *toolButtonsLayout = new QGridLayout(buttonsGroupBox);

    QAction *num1Action = new QAction(tr("1"),this);
    connect(num1Action, SIGNAL(triggered()), this, SLOT(insertNum1()));
    addAction(num1Action);
    QAction *num2Action = new QAction(tr("2"),this);
    connect(num2Action, SIGNAL(triggered()), this, SLOT(insertNum2()));
    addAction(num2Action);
    QAction *num3Action = new QAction(tr("3"),this);
    connect(num3Action, SIGNAL(triggered()), this, SLOT(insertNum3()));
    addAction(num3Action);
    QAction *num4Action = new QAction(tr("4"),this);
    connect(num4Action, SIGNAL(triggered()), this, SLOT(insertNum4()));
    addAction(num4Action);
    QAction *num5Action = new QAction(tr("5"),this);
    connect(num5Action, SIGNAL(triggered()), this, SLOT(insertNum5()));
    addAction(num5Action);
    QAction *num6Action = new QAction(tr("6"),this);
    connect(num6Action, SIGNAL(triggered()), this, SLOT(insertNum6()));
    addAction(num6Action);
    QAction *num7Action = new QAction(tr("7"),this);
    connect(num7Action, SIGNAL(triggered()), this, SLOT(insertNum7()));
    addAction(num7Action);
    QAction *num8Action = new QAction(tr("8"),this);
    connect(num8Action, SIGNAL(triggered()), this, SLOT(insertNum8()));
    addAction(num8Action);
    QAction *num9Action = new QAction(tr("9"),this);
    connect(num9Action, SIGNAL(triggered()), this, SLOT(insertNum9()));
    addAction(num9Action);
    QAction *num0Action = new QAction(tr("0"),this);
    connect(num0Action, SIGNAL(triggered()), this, SLOT(insertNum0()));
    addAction(num0Action);

    QAction *divideAction = new QAction(tr("/"),this);
    connect(divideAction, SIGNAL(triggered()), this, SLOT(qtCalcDivide()));
    addAction(divideAction);

    QAction *timesAction = new QAction(tr("*"),this);
    connect(timesAction, SIGNAL(triggered()), this, SLOT(qtCalcTimes()));
    addAction(timesAction);

    QAction *minusAction = new QAction(tr("-"),this);
    connect(minusAction, SIGNAL(triggered()), this, SLOT(qtCalcMinus()));
    addAction(minusAction);

    QAction *plusAction = new QAction(tr("+"),this);
    connect(plusAction, SIGNAL(triggered()), this, SLOT(qtCalcPlus()));
    addAction(plusAction);

    QAction *commaAction = new QAction(tr("."),this);
    connect(commaAction, SIGNAL(triggered()), this, SLOT(qtCalcComma()));
    addAction(commaAction);

    QAction *clearAction = new QAction(tr("C"),this);
    connect(clearAction, SIGNAL(triggered()), formulaEdit, SLOT(clear()));
    addAction(clearAction);

    QAction *braceOpenAction = new QAction(tr("("),this);
    connect(braceOpenAction, SIGNAL(triggered()), this, SLOT(qtCalcBraceOpen()));
    addAction(braceOpenAction);

    QAction *braceCloseAction = new QAction(tr(")"),this);
    connect(braceCloseAction, SIGNAL(triggered()), this, SLOT(qtCalcBraceClose()));
    addAction(braceCloseAction);


    QToolButton *num1 = new QCalculatorToolButton(buttonsGroupBox);
    num1->setDefaultAction(num1Action);
    QToolButton *num2 = new QCalculatorToolButton(buttonsGroupBox);
    num2->setDefaultAction(num2Action);
    QToolButton *num3 = new QCalculatorToolButton(buttonsGroupBox);
    num3->setDefaultAction(num3Action);
    QToolButton *num4 = new QCalculatorToolButton(buttonsGroupBox);
    num4->setDefaultAction(num4Action);
    QToolButton *num5 = new QCalculatorToolButton(buttonsGroupBox);
    num5->setDefaultAction(num5Action);
    QToolButton *num6 = new QCalculatorToolButton(buttonsGroupBox);
    num6->setDefaultAction(num6Action);
    QToolButton *num7 = new QCalculatorToolButton(buttonsGroupBox);
    num7->setDefaultAction(num7Action);
    QToolButton *num8 = new QCalculatorToolButton(buttonsGroupBox);
    num8->setDefaultAction(num8Action);
    QToolButton *num9 = new QCalculatorToolButton(buttonsGroupBox);
    num9->setDefaultAction(num9Action);
    QToolButton *num0 = new QCalculatorToolButton(buttonsGroupBox);
    num0->setDefaultAction(num0Action);
    QToolButton *divideButton = new QCalculatorToolButton(buttonsGroupBox);
    divideButton->setDefaultAction(divideAction);
    QToolButton *timesButton = new QCalculatorToolButton(buttonsGroupBox);
    timesButton->setDefaultAction(timesAction);
    QToolButton *minusButton = new QCalculatorToolButton(buttonsGroupBox);
    minusButton->setDefaultAction(minusAction);
    QToolButton *plusButton = new QCalculatorToolButton(buttonsGroupBox);
    plusButton->setDefaultAction(plusAction);
    QToolButton *commaButton = new QCalculatorToolButton(buttonsGroupBox);
    commaButton->setDefaultAction(commaAction);
    QToolButton *clearButton = new QCalculatorToolButton(buttonsGroupBox);
    clearButton->setDefaultAction(clearAction);
    QToolButton *braceOpenButton = new QCalculatorToolButton(buttonsGroupBox);
    braceOpenButton->setDefaultAction(braceOpenAction);
    QToolButton *braceCloseButton = new QCalculatorToolButton(buttonsGroupBox);
    braceCloseButton->setDefaultAction(braceCloseAction);

    toolButtonsLayout->addWidget(num7, 0, 0);
    toolButtonsLayout->addWidget(num8, 0, 1);
    toolButtonsLayout->addWidget(num9, 0, 2);

    toolButtonsLayout->addWidget(num4, 1, 0);
    toolButtonsLayout->addWidget(num5, 1, 1);
    toolButtonsLayout->addWidget(num6, 1, 2);

    toolButtonsLayout->addWidget(num1, 2, 0);
    toolButtonsLayout->addWidget(num2, 2, 1);
    toolButtonsLayout->addWidget(num3, 2, 2);

    toolButtonsLayout->addWidget(num0, 3, 0);
    toolButtonsLayout->addWidget(commaButton, 3, 1);
    toolButtonsLayout->addWidget(plusButton, 3, 2);

    toolButtonsLayout->addWidget(minusButton, 4, 0);
    toolButtonsLayout->addWidget(timesButton, 4, 1);
    toolButtonsLayout->addWidget(divideButton, 4, 2);

    toolButtonsLayout->addWidget(braceOpenButton, 5, 0);
    toolButtonsLayout->addWidget(braceCloseButton, 5, 1);
    toolButtonsLayout->addWidget(clearButton, 5, 2);

    svgLayout->addWidget(buttonsGroupBox);
    svgLayout->addStretch(1);

    mainLayout->addLayout(svgLayout);

    QPushButton *okButton = new QPushButton(QIcon(":images/ok.svg"),tr("OK"), this);
    QPushButton *cancelButton = new QPushButton(QIcon(":images/cancel.svg"),tr("Cancel"), this);

    QHBoxLayout *buttonLayout = new QHBoxLayout;

    buttonLayout->addStretch(1);
    buttonLayout->addWidget(okButton);
    buttonLayout->addWidget(cancelButton);

    //connect(okButton, SIGNAL(clicked()), this, SLOT(verifyFairyInputLines()));
    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

    verticalLayout->addLayout(buttonLayout);

    mainLayout->addLayout(verticalLayout);

    setLayout(mainLayout);

    editareGroupBox->setTitle(QApplication::translate("Dialog", "Suporta operatiile elementare si paranteze", 0));
    optionsGroupBox->setTitle(QApplication::translate("Dialog", "Optiuni", 0));

    formulaLabel->setText(QApplication::translate("Dialog", "&Formula", 0));
    resultLabel->setText(QApplication::translate("Dialog", "&Rezultat", 0));
    decimalLabel->setText(QApplication::translate("Dialog", "&Numar zecimale", 0));

    connect(formulaEdit, SIGNAL(textChanged()), this, SLOT(refreshResult()));
    connect(thousandCheckBox, SIGNAL(stateChanged(int)), this, SLOT(refreshResult()));
    connect(decimalSpinBox, SIGNAL(valueChanged(int)), this, SLOT(refreshResult()));
    connect(formulaEdit, SIGNAL(okSignal()), this, SLOT(accept()));

    refreshResult();
    formulaEdit->setFocus();
}

void QCalculatorDialog::setFormula(const QString &formula)
{
    formulaEdit->setText(formula);
}

QString QCalculatorDialog::getFormula()
{
    return formulaEdit->toPlainText().simplified();
}

QString QCalculatorDialog::getValue()
{
    QString result;
    int i = decimalSpinBox->value();

    result = QString::number(d,'f',i);

    return result;
}

void QCalculatorDialog::refreshResult()
{
    QString result;
    QString resultForParent;
    QLocale clocale(QLocale::German);
    formula = formulaEdit->toPlainText().simplified().remove(" ");
    p = 0;

    d = eval();
    p = 0;

    resultForParent = QString::number(d,'f',decimalSpinBox->value());
    if (thousandCheckBox->isChecked())
    {
        result = clocale.toString(d,'f',decimalSpinBox->value());
    }
    else
    {
        result = QString::number(d,'f',decimalSpinBox->value());
    }

    resultEdit->setText(result);
    emit setParentText(resultForParent);
}

double QCalculatorDialog::eval()
{
    double r = termen();
    while ( (formula.data()[p] =='+') || (formula.data()[p]=='-') )
    {
        if (formula.data()[p] == '+')
        {
            ++p;
            r += termen();
        }
        if (formula.data()[p] == '-')
        {
            ++p;
            r -= termen();
        }
    }

    return r;
}

double QCalculatorDialog::termen()
{
    double r = factor();
    while ((formula.data()[p] == '*') || (formula.data()[p] == '/'))
    {
        if (formula.data()[p] == '*')
        {
            ++p;
            r *= factor();
        }
        if (formula.data()[p] == '/')
        {
            ++p;
            r /= factor();
        }
    }
    return r;
}

double QCalculatorDialog::factor()
{
    double r=0;
    double t=0;
    int i = 0;
    QString convert;
    bool ok;
    if (formula.data()[p] == '(')
    {
        ++p;
        r = eval();
        ++p;
    }
    else
    {
        while ( (formula.data()[p]>='0') && (formula.data()[p]<='9') )
        {
            convert = formula[p];
            r = r*10 + convert.toInt(&ok, 10);
            ++p;
        }
        if (formula.data()[p] == '.')
        {
            ++p;
            i = 0;
            while ( (formula.data()[p] >='0') && (formula.data()[p]<='9') )
            {
                convert = formula[p];
                t = t*10 + convert.toInt(&ok, 10);
                ++p;
                ++i;
            }
        }
        long x = 1;
        for (int j=0;j<i;j++)
        {
            x *= 10;
        }
        r = r + t/x;
    }
    return r;
}

void QCalculatorDialog::insertNum1()
{
    formulaEdit->insertPlainText("1");
}

void QCalculatorDialog::insertNum2()
{
    formulaEdit->insertPlainText("2");
}

void QCalculatorDialog::insertNum3()
{
    formulaEdit->insertPlainText("3");
}

void QCalculatorDialog::insertNum4()
{
    formulaEdit->insertPlainText("4");
}

void QCalculatorDialog::insertNum5()
{
    formulaEdit->insertPlainText("5");
}

void QCalculatorDialog::insertNum6()
{
    formulaEdit->insertPlainText("6");
}

void QCalculatorDialog::insertNum7()
{
    formulaEdit->insertPlainText("7");
}

void QCalculatorDialog::insertNum8()
{
    formulaEdit->insertPlainText("8");
}

void QCalculatorDialog::insertNum9()
{
    formulaEdit->insertPlainText("9");
}

void QCalculatorDialog::insertNum0()
{
    formulaEdit->insertPlainText("0");
}

void QCalculatorDialog::qtCalcDivide()
{
    formulaEdit->insertPlainText("/");
}

void QCalculatorDialog::qtCalcTimes()
{
    formulaEdit->insertPlainText("*");
}

void QCalculatorDialog::qtCalcPlus()
{
    formulaEdit->insertPlainText("+");
}

void QCalculatorDialog::qtCalcMinus()
{
    formulaEdit->insertPlainText("-");
}

void QCalculatorDialog::qtCalcComma()
{
    formulaEdit->insertPlainText(".");
}

void QCalculatorDialog::qtCalcBraceOpen()
{
    formulaEdit->insertPlainText("(");
}

void QCalculatorDialog::qtCalcBraceClose()
{
    formulaEdit->insertPlainText(")");
}

void QCalculatorDialog::readSettings()
{
    QSettings settings;
    formula = settings.value("QCalculatorDialog/formula", "").toString();
    numberOfDecimals = settings.value("QCalculatorDialog/numberofdecimals",2).toInt();
    thousandSeparator = settings.value("QCalculatorDialog/thousandseparator",true).toBool();
}

void QCalculatorDialog::writeSettings()
{
    QSettings settings;
    settings.setValue("QCalculatorDialog/formula", getFormula());
    settings.setValue("QCalculatorDialog/numberofdecimals",decimalSpinBox->value());
    settings.setValue("QCalculatorDialog/thousandseparator",thousandCheckBox->isChecked());
}

//QCalculatorToolButton

QCalculatorToolButton::QCalculatorToolButton(QWidget *parent)
    : QToolButton(parent)
{
    QFont font = this->font();
    font.setBold(true);
    font.setPointSize(font.pointSize()+2);
    setFont(font);
}

QSize QCalculatorToolButton::sizeHint() const
{
    QSize size = QToolButton::sizeHint();
    size.rheight() += 10;
    size.rwidth() = qMax(size.width(), size.height());
    return size;
}

void QCalculatorToolButton::setIconSize1(int i)
{
    QSize size = QSize(i,i);
    setIconSize(size);
}
