/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef __CALCULATOR_H
#define __CALCULATOR_H

#include <QTextEdit>

#include "widgets.h"

class QCheckBox;
class QSpinBox;
class QFairyLineEdit;

//QCalculatorFormulaEdit

class QCalculatorFormulaEdit : public QTextEdit
{
    Q_OBJECT

public:
    QCalculatorFormulaEdit(const QString &text, QWidget *parent = 0);

signals:
     void okSignal();

private:
     void keyPressEvent(QKeyEvent *);

};

//QCalculatorDialog

class QCalculatorDialog : public QDialog
{
    Q_OBJECT

public:
    QCalculatorDialog(bool a_fromLineEdit, const QString &a_objectName, const QString &title, QWidget *parent = 0);

public slots:
    void refreshResult();

public:
    void setFormula(const QString &formula);
    QString getFormula();
    QString getValue();

signals:
    void setParentText(QString);

private slots:
    void insertNum1();
    void insertNum2();
    void insertNum3();
    void insertNum4();
    void insertNum5();
    void insertNum6();
    void insertNum7();
    void insertNum8();
    void insertNum9();
    void insertNum0();
    void qtCalcDivide();
    void qtCalcTimes();
    void qtCalcPlus();
    void qtCalcMinus();
    void qtCalcComma();
    void qtCalcBraceOpen();
    void qtCalcBraceClose();

private:
    void readSettings();
    void writeSettings();

    QTextEdit *formulaEdit;
    QFairyLineEdit *resultEdit;

    QCheckBox *thousandCheckBox;
    QSpinBox *decimalSpinBox;

    double termen();
    double factor();
    double eval();
    bool thousandSeparator;
    int numberOfDecimals;

    double d;

    int p;
    QString formula;
};

//QCalculatorToolButton

class QCalculatorToolButton : public QToolButton
{
    Q_OBJECT

public:
    QCalculatorToolButton(QWidget *parent = 0);

    QSize sizeHint() const;

public slots:
    void setIconSize1(int);
};

#endif //__CALCULATOR_H
