/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QLabel>
#include <QGridLayout>
#include <QSettings>
#include <QGroupBox>

#include "finddialog.h"
#include "lineedits.h"
#include "global.h"

//QFairyFindDialog

QFairyFindDialog::QFairyFindDialog(const QString &a_objectName, const QString &title, QWidget *parent) :
    QFairyModalDialog(a_objectName, title, parent)
{
    QSettings settings;
    QString search = settings.value("finddialog/search", "").toString();

    QGroupBox *inputGroupBox = new QGroupBox(this);
    QVBoxLayout *inputLayout = new QVBoxLayout(inputGroupBox);

    QLabel *findStringLabel = new QLabel(tr("Text to find"), inputGroupBox);
    findStringLineEdit = new QFairyLineEdit(search, inputGroupBox);
    findStringLabel->setBuddy(findStringLineEdit);

    inputLayout->addWidget(findStringLabel);
    inputLayout->addWidget(findStringLineEdit);

    QHBoxLayout *buttonLayout = new QHBoxLayout;

    QPushButton *findButton = new QPushButton();
    findButton->setIcon(QIcon(":images/search.svg"));
    findButton->setIconSize(QSize(27,27));
    findButton->setDefault(true);

    QPushButton *closeButton = new QPushButton();
    closeButton->setIcon(QIcon(":images/exit.svg"));
    closeButton->setIconSize(QSize(27,27));

    buttonLayout->addStretch(1);
    buttonLayout->addWidget(closeButton);
    buttonLayout->addWidget(findButton);

    connect(findButton, SIGNAL(clicked()), this, SLOT(find()));
    connect(closeButton, SIGNAL(clicked()), this, SLOT(hide()));

    QVBoxLayout *mainLayout = new QVBoxLayout;

    mainLayout->addWidget(inputGroupBox);
    mainLayout->addStretch(1);
    mainLayout->addLayout(buttonLayout);

    setLayout(mainLayout);
    setWindowTitle(title);

    findStringLineEdit->setFocus();
}

void QFairyFindDialog::find()
{
    emit findFirst(findStringLineEdit->text());
}

QFairyFindDialog::~QFairyFindDialog()
{
    QSettings settings;
    settings.setValue("finddialog/search", findStringLineEdit->text());
}
