/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QApplication>
#include <QSettings>
#include <QWebView>
#include <QWebPage>
#include <QWebFrame>
#include <QAction>
#include <QVBoxLayout>
#include <QFontDialog>
#include <QFileInfo>
#include <QPrinter>
#include <QPrintDialog>
#include <QPrintPreviewDialog>
#include <QFileDialog>
#include <QInputDialog>
#include <QMessageBox>
#include <QKeyEvent>
#include <QPlainTextEdit>
#include <QSqlQuery>
#include <QSqlDatabase>
#include <QSqlError>
#include <QSettings>
#include <QTextCursor>
#include <QMenu>
#include <QSvgWidget>

#include "highlighter.h"
#include "global.h"
#include "textedit.h"
#include "widgets.h"
#include "formulaeditor.h"

//QFairyFormulaEditor

QFairyFormulaEditor::QFairyFormulaEditor(const QString &a_text, const QString &title, QWidget *parent)
    : QFairyModalDialog("QFairyFormulaEditor", title, parent)
{
    editValue = a_text;
    saveOrNot = false;

    setWindowIcon(QIcon(":images/formule.svg"));

    textEdit = new QFairyTextEdit(this);

    QAction *saveAction = new QAction(QIcon(":images/save.svg"),tr("Save"), this);
    connect(saveAction, SIGNAL(triggered()), this, SLOT(save()));
    textEdit->addAction(saveAction);

    saveButton = new QFairyToolButton(this);
    saveButton->setFocusPolicy(Qt::NoFocus);
    saveButton->setDefaultAction(saveAction);
    saveButton->setIconSize(QSize(51,51));

    QVBoxLayout *buttonLayout = new QVBoxLayout;

    buttonLayout->addWidget(saveButton);
    buttonLayout->addStretch(1);

    setWindowTitle(title);

    QSvgWidget *picture = new QSvgWidget(":images/formule.svg", this);
    picture->setFixedSize(QSize(128,128));

    QVBoxLayout *svgLayout = new QVBoxLayout;
    svgLayout->addWidget(picture);
    svgLayout->addStretch(1);

    QHBoxLayout *mainLayout = new QHBoxLayout(this);
    mainLayout->addLayout(svgLayout);
    mainLayout->addWidget(textEdit);
    mainLayout->addLayout(buttonLayout);
    setLayout(mainLayout);

    //connect(parent, SIGNAL(setToolButtonIconSize(int)), this, SLOT(setToolButtonIconSize(int)));

    textEdit->setPlainText(editValue);
    textEdit->moveCursor(QTextCursor::Start);
    textEdit->setLineWrapMode(QTextEdit::NoWrap);
    QFairyHighlighter *highlighter = new QFairyHighlighter(textEdit->document());
    textEdit->document()->setModified(false);
}

bool QFairyFormulaEditor::textChanged()
{
    return saveOrNot;
}

void QFairyFormulaEditor::save()
{
    textEdit->document()->setModified(false);
    saveOrNot = true;
    close();
}

QString QFairyFormulaEditor::getText()
{
    return textEdit->document()->toPlainText();
}

void QFairyFormulaEditor::keyPressEvent(QKeyEvent *event)
{
    switch (event->key())
    {
    case Qt::Key_Escape:
        close();
        break;
    default:
        QWidget::keyPressEvent(event);
    }
}

void QFairyFormulaEditor::closeEvent(QCloseEvent *event)
{
    if (textEdit->document()->isModified())
    {
        QMessageBox::StandardButton ret;
        ret = QMessageBox::warning(this, tr("Save document"),tr("The document has been modified.\n"
                                                                   "Do you want to save your changes?"),
                                   QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
        if (ret == QMessageBox::Save)
        {
            saveOrNot = true; //save();
            event->accept();
        }
        else if (ret == QMessageBox::Cancel)
        {
            event->ignore();
        }
        else
        {
            saveOrNot = false;
            //textEdit->document()->setModified(false);
            event->accept();
        }
    }

    //writeSettings();
}
