/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/
#include <QSettings>
#include <QDialog>
#include <QVBoxLayout>
#include <QSqlQueryModel>
#include <QTableView>
#include <QSqlRecord>
#include <QPushButton>
#include <QSqlQuery>
#include <QMessageBox>
#include <QSqlError>

#include "highlighter.h"
#include "formatcolorglobals.h"

//QFairyHighlighter

QFairyHighlighter::QFairyHighlighter(QTextDocument *parent)
    : QSyntaxHighlighter(parent)
{
    QByteArray tempByteArray;
    quint8 r,g,b;

    HighlightingRule rule;

    tempByteArray.clear();
    tempByteArray.append(keywordFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor keywordFormatColor(r,g,b);
    keywordFormat.setForeground(keywordFormatColor);
    keywordFormat.setFontWeight(QFont::Bold);
    keywordFormat.setFontCapitalization(QFont::AllUppercase);
    QStringList keywordPatterns;
    keywordPatterns
                    << "\\bbigint\\b"         << "\\bbinary\\b"          << "\\bbit\\b"
                    << "\\bblob\\b"           << "\\bbool\\b"            << "\\bboolean\\b"       << "\\bchar\\b"
                    << "\\bdate\\b"           << "\\bdatetime\\b"        << "\\bdecimal\\b"       << "\\bdouble\\b"
                    << "\\benum\\b"           << "\\bfloat\\b"           << "\\bint\\b"           << "\\blongblob\\b"
                    << "\\bfloat\\b"          << "\\bint\\b"             << "\\blongblob\\b"      << "\\blongtext\\b"
                    << "\\bmediumblob\\b"     << "\\bmediumint\\b"       << "\\bmediumtext\\b"
                    << "\\bnumeric\\b"        << "\\breal\\b"            << "\\bset\\b"           << "\\bsmallint\\b"
                    << "\\btext\\b"           << "\\btime\\b"            << "\\btimestamp\\b"     << "\\btinyblob\\b"
                    << "\\btinyint\\b"        << "\\btinytext\\b"        << "\\bvarbinary\\b"
                    << "\\bselect\\b"         << "\\bfrom\\b"            << "\\bwhere\\b"         << "\\border\\b"
                    << "\\bvarchar\\b"        << "\\byear\\b"
    << "\\bBIGINT\\b"         << "\\bbinary\\b"          << "\\bbit\\b"
    << "\\bBLOB\\b"           << "\\bbool\\b"            << "\\bboolean\\b"       << "\\bchar\\b"
    << "\\bDATE\\b"           << "\\bdatetime\\b"        << "\\bdecimal\\b"       << "\\bdouble\\b"
    << "\\bENUM\\b"           << "\\bfloat\\b"           << "\\bint\\b"           << "\\blongblob\\b"
    << "\\bFLOAT\\b"          << "\\bint\\b"             << "\\blongblob\\b"      << "\\blongtext\\b"
    << "\\bMEDIUMBLOB\\b"     << "\\bmediumint\\b"       << "\\bmediumtext\\b"
    << "\\bNUMERIC\\b"        << "\\breal\\b"            << "\\bset\\b"           << "\\bsmallint\\b"
    << "\\bTEXT\\b"           << "\\btime\\b"            << "\\btimestamp\\b"     << "\\btinyblob\\b"
    << "\\bTINYINT\\b"        << "\\btinytext\\b"        << "\\bvarbinary\\b"
    << "\\bSELECT\\b"         << "\\bFROM\\b"            << "\\bWHERE\\b"         << "\\bORDER\\b" << "\\bINSERT\\b"<< "\\bUPDATE\\b"<< "\\bDELETE\\b"
    << "\\bVARCHAR\\b"        << "\\byear\\b"<< "\\bVALUES\\b"<< "\\bINTO\\b"<< "\\bSET\\b"
       << "\\bDROP\\b" << "\\bTABLE\\b" << "\\bIF\\b" << "\\bEXISTS\\b" << "\\bCREATE\\b" << "\\bSCHEMA\\b" << "\\bDATABASE\\b" << "\\bUSE\\b"
       << "\\bNOT\\b"<< "\\bNULL\\b"<< "\\bDEFAULT\\b"  << "\\bENGINE\\b"<< "\\bCHARSET\\b" ;

    foreach (const QString &pattern, keywordPatterns)
    {
        rule.pattern = QRegExp(pattern);
        rule.format = keywordFormat;
        rule.format.setFontCapitalization(QFont::AllUppercase);
        highlightingRules.append(rule);
    }

    tempByteArray.clear();
    tempByteArray.append(classFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor classFormatColor(r,g,b);
    classFormat.setForeground(classFormatColor);
    classFormat.setFontWeight(QFont::Bold);
    rule.pattern = QRegExp("^\\$\\$[^\n]*");
    rule.format = classFormat;
    highlightingRules.append(rule);

    tempByteArray.clear();
    tempByteArray.append(singleLineCommentFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor singleLineCommentFormatColor(r,g,b);
    singleLineCommentFormat.setForeground(singleLineCommentFormatColor);
    rule.pattern = QRegExp("^#[^\n]*");
    rule.format = singleLineCommentFormat;
    highlightingRules.append(rule);

    tempByteArray.clear();
    tempByteArray.append(quotationFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor quotationFormatColor(r,g,b);
    quotationFormat.setForeground(quotationFormatColor);
    rule.pattern = QRegExp("<[A-Za-z0-9\\s/='%]+>");
    rule.format = quotationFormat;
    highlightingRules.append(rule);

    tempByteArray.clear();
    tempByteArray.append(functionFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor functionFormatColor(r,g,b);
    functionFormat.setFontItalic(true);
    functionFormat.setForeground(functionFormatColor);
    rule.pattern = QRegExp("\\b[A-Za-z0-9_]+(?=\\()");
    rule.format = functionFormat;
    highlightingRules.append(rule);

    tempByteArray.clear();
    tempByteArray.append(showFieldFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor showFieldFormatColor(r,g,b);
    showFieldFormat.setForeground(showFieldFormatColor);
    rule.pattern = QRegExp("\\{[A-Za-z0-9\\s/='']+\\}");
    rule.format = showFieldFormat;
    highlightingRules.append(rule);

    tempByteArray.clear();
    tempByteArray.append(showFieldFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor ashowFieldFormatColor(r,g,b);
    showFieldFormat.setForeground(ashowFieldFormatColor);
    rule.pattern = QRegExp("\\%[A-Za-z0-9_\\s/='']+\\%");
    rule.format = showFieldFormat;
    highlightingRules.append(rule);

    tempByteArray.clear();
    tempByteArray.append(showFieldFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor ashow1FieldFormatColor(r,g,b);
    showFieldFormat.setForeground(ashow1FieldFormatColor);
    rule.pattern = QRegExp("\\`[A-Za-z0-9_\\s/='']+\\`");
    rule.format = showFieldFormat;
    highlightingRules.append(rule);

    tempByteArray.clear();
    tempByteArray.append(htmlNumberFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor htmlNumberFormatColor(r,g,b);
    htmlNumberFormat.setForeground(htmlNumberFormatColor);
    rule.pattern = QRegExp("&[A-Za-z0-9\\s/=''#]+;");
    rule.format = htmlNumberFormat;
    highlightingRules.append(rule);

    tempByteArray.clear();
    tempByteArray.append(dollarNumberFormatColorString);
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor dollarNumberFormatColor(r,g,b);
    dollarNumberFormat.setForeground(dollarNumberFormatColor);
    rule.pattern = QRegExp("\\s\\$[A-Za-z]");
    rule.format = dollarNumberFormat;
    highlightingRules.append(rule);

    tempByteArray.clear();
    tempByteArray.append("008000");
    tempByteArray = QByteArray::fromHex(tempByteArray);
    r = tempByteArray.at(0);
    g = tempByteArray.at(1);
    b = tempByteArray.at(2);
    QColor stringFormatColor(r,g,b);
    stringFormat.setForeground(stringFormatColor);
    rule.pattern = QRegExp("'[A-Za-z0-9%\\s\\\\]+'");
    rule.format = stringFormat;
    highlightingRules.append(rule);

//commentStartExpression = QRegExp("/\\*");
//commentEndExpression = QRegExp("\\*/");
}
/*
void QFairyHighlighter::highlightBlock(const QString &text)
{
    foreach (const HighlightingRule &rule, highlightingRules)
        {
            QRegExp expression(rule.pattern);
            int index = expression.indexIn(text);
            while (index >= 0)
                {
                    int length = expression.matchedLength();
                setFormat(index, length, rule.format);
                index = expression.indexIn(text, index + length);
                }
        }

    setCurrentBlockState(0);

    int startIndex = 0;
    if (previousBlockState() != 1)
        startIndex = commentStartExpression.indexIn(text);


    while (startIndex >= 0)
        {
            int endIndex = commentEndExpression.indexIn(text, startIndex);
            int commentLength;
            if (endIndex == -1)
                {
                    setCurrentBlockState(1);
                    commentLength = text.length() - startIndex;
                }
            else
                {
                    commentLength = endIndex - startIndex + commentEndExpression.matchedLength();
                }

            //setFormat(startIndex, commentLength, multiLineCommentFormat);
            startIndex = commentStartExpression.indexIn(text, startIndex + commentLength);
        }
}
*/

void QFairyHighlighter::highlightBlock(const QString &text)
{
    foreach (const HighlightingRule &rule, highlightingRules)
    {
        QRegExp expression(rule.pattern);
        int index = expression.indexIn(text);
        while (index >= 0)
        {
            int length = expression.matchedLength();
            setFormat(index, length, rule.format);
            index = expression.indexIn(text, index + length);
        }
    }
}

