/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QSettings>
#include <QVBoxLayout>
#include <QGroupBox>
#include <QLineEdit>

#include "global.h"
#include "keyboard.h"

//QFairyKeyboard

QFairyKeyboard::QFairyKeyboard(const QString &a_objectName, const QString &content, const QString &title, QWidget *parent)
    : QFairyModalDialog(a_objectName, title, parent)
{
    capsLockPressed = true;

    QSettings settings;
    capsLockPressed = settings.value("fairykeyboard/capslock", false).toBool();

    QVBoxLayout *mainLayout = new QVBoxLayout;


    QGroupBox *editareGroupBox = new QGroupBox(this);

    QVBoxLayout *editareLayout = new QVBoxLayout(editareGroupBox);

    QHBoxLayout *numericRowLayout = new QHBoxLayout;
    QHBoxLayout *firstRowLayout = new QHBoxLayout;
    QHBoxLayout *secondRowLayout = new QHBoxLayout;
    QHBoxLayout *thirdRowLayout = new QHBoxLayout;
    QHBoxLayout *localRowLayout = new QHBoxLayout;
    QHBoxLayout *spaceRowLayout = new QHBoxLayout;
    QHBoxLayout *buttonLayout = new QHBoxLayout;

    keyboardLine = new QLineEdit(content, editareGroupBox);
    QFont font = keyboardLine->font();
    font.setPointSize(font.pointSize() + editFontPlus);
    keyboardLine->setFont(font);

    QToolButton *capsLockButton = new QFairyKeyboardToolButton("Caps lock",this);
    connect(capsLockButton, SIGNAL(clicked()), this, SLOT(capsLockClicked()));
    QToolButton *backspaceButton = new QFairyKeyboardToolButton("<-",this);
    connect(backspaceButton, SIGNAL(clicked()), this, SLOT(backspaceClicked()));

    for (int i=0;i<10;i++)
    {
        charButtons.append(createButton(QString::number(i)));
        numericRowLayout->addWidget(charButtons.at(i));
    }

    charButtons.append(createButton("Q"));
    charButtons.append(createButton("W"));
    charButtons.append(createButton("E"));
    charButtons.append(createButton("R"));
    charButtons.append(createButton("T"));
    charButtons.append(createButton("Y"));
    charButtons.append(createButton("U"));
    charButtons.append(createButton("I"));
    charButtons.append(createButton("O"));
    charButtons.append(createButton("P"));

    charButtons.append(createButton("A"));
    charButtons.append(createButton("S"));
    charButtons.append(createButton("D"));
    charButtons.append(createButton("F"));
    charButtons.append(createButton("G"));
    charButtons.append(createButton("H"));
    charButtons.append(createButton("J"));
    charButtons.append(createButton("K"));
    charButtons.append(createButton("L"));
    charButtons.append(createButton("*"));
    charButtons.append(createButton("`"));

    charButtons.append(createButton("Z"));
    charButtons.append(createButton("X"));
    charButtons.append(createButton("C"));
    charButtons.append(createButton("V"));
    charButtons.append(createButton("B"));
    charButtons.append(createButton("N"));
    charButtons.append(createButton("M"));
    charButtons.append(createButton(","));
    charButtons.append(createButton("."));

    if ((fairyLocal == "ro") || (fairyLocal == "hu"))
    {
        charButtons.append(createButton(QString::fromUtf8("Ă")));
        charButtons.append(createButton(QString::fromUtf8("Â")));
        charButtons.append(createButton(QString::fromUtf8("Î")));
        charButtons.append(createButton(QString::fromUtf8("Ş")));
        charButtons.append(createButton(QString::fromUtf8("Ţ")));
        //}

        //if (fairyLocal == "hu")
        //{
        charButtons.append(createButton(QString::fromUtf8("Á")));
        charButtons.append(createButton(QString::fromUtf8("É")));
        charButtons.append(createButton(QString::fromUtf8("Í")));
        charButtons.append(createButton(QString::fromUtf8("Ó")));
        charButtons.append(createButton(QString::fromUtf8("Ö")));
        charButtons.append(createButton(QString::fromUtf8("Ő")));
        charButtons.append(createButton(QString::fromUtf8("Ú")));
        charButtons.append(createButton(QString::fromUtf8("Ü")));
        charButtons.append(createButton(QString::fromUtf8("Ű")));
    }

    charButtons.append(createButton(" "));

    int i;

    firstRowLayout->addWidget(capsLockButton);
    for (i=10;i<20;i++) firstRowLayout->addWidget(charButtons[i]);
    firstRowLayout->addWidget(backspaceButton);
    for (i=20;i<31;i++) secondRowLayout->addWidget(charButtons[i]);
    for (i=31;i<40;i++) thirdRowLayout->addWidget(charButtons[i]);
    if ((fairyLocal == "ro") || (fairyLocal == "hu"))
    {
        for (i=40;i<54;i++) localRowLayout->addWidget(charButtons[i]);
    }

    //if (fairyLocal == "hu")
    //{
    //for (i=40;i<49;i++) localRowLayout->addWidget(charButtons[i]);
    //}

    spaceRowLayout->addWidget(charButtons[i]);

    QPushButton *okButton = new QPushButton(QIcon(":images/ok.svg"),tr("OK"), this);
    QPushButton *cancelButton = new QPushButton(QIcon("images/cancel.svg"),tr("Cancel"), this);

    connect(okButton, SIGNAL(clicked()), this, SLOT(accept()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

    buttonLayout->addStretch(1);
    buttonLayout->addWidget(okButton);
    buttonLayout->addWidget(cancelButton);

    editareLayout->addWidget(keyboardLine);
    editareLayout->addLayout(numericRowLayout);
    editareLayout->addLayout(firstRowLayout);
    editareLayout->addLayout(secondRowLayout);
    editareLayout->addLayout(thirdRowLayout);
    editareLayout->addLayout(localRowLayout);
    editareLayout->addLayout(spaceRowLayout);
    editareLayout->addLayout(buttonLayout);

    mainLayout->addWidget(editareGroupBox);

    setLayout(mainLayout);

    if (!capsLockPressed)
        for (int i=0;i<charButtons.count();i++)
            charButtons[i]->setText(charButtons.at(i)->text().toLower());
}

QFairyKeyboardToolButton *QFairyKeyboard::createButton(const QString &text)
{
    QFairyKeyboardToolButton *button = new QFairyKeyboardToolButton(text);
    connect(button, SIGNAL(clicked()), this, SLOT(buttonPressed()));
    QFont font = button->font();
    font.setPointSize(font.pointSize() + editFontPlus);
    button->setFont(font);

    return button;
}

void QFairyKeyboard::capsLockClicked()
{
    if (capsLockPressed)
        for (int i=0;i<charButtons.count();i++)
            charButtons[i]->setText(charButtons.at(i)->text().toLower());
    else
        for (int i=0;i<charButtons.count();i++)
            charButtons[i]->setText(charButtons.at(i)->text().toUpper());

    capsLockPressed = !capsLockPressed;
}

void QFairyKeyboard::backspaceClicked()
{
    keyboardLine->backspace();
}

QString QFairyKeyboard::getValue()
{
    return keyboardLine->text();
}

void QFairyKeyboard::buttonPressed()
{
    QFairyKeyboardToolButton *clickedButton = qobject_cast<QFairyKeyboardToolButton *>(sender());
    //emit buttonPressedSignal(clickedButton->text());
    //QString t = keyboardLine->text();
    //keyboardLine->setText(t+clickedButton->text());
    keyboardLine->insert(clickedButton->text());
}

void QFairyKeyboard::writeSettings()
{
    QSettings settings;
    settings.setValue("fairykeyboard/capslock", capsLockPressed);
}

//QFairyKeyboardToolButton

QFairyKeyboardToolButton::QFairyKeyboardToolButton(const QString &text, QWidget *parent)
    : QToolButton(parent)
{
    setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
    setText(text);
}

QSize QFairyKeyboardToolButton::sizeHint() const

{
    QSize size = QToolButton::sizeHint();
    size.rheight() += 2;
    size.rwidth() = qMax(size.width(), size.height());
    return size;
}
