/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#ifndef __LINEEDITS_H
#define __LINEEDITS_H

#include <QLineEdit>

//QReadOnlyLineEdit

class QReadOnlyLineEdit : public QLineEdit
{
    Q_OBJECT

public:
    QReadOnlyLineEdit(const QString &contents, QWidget *parent = 0);
};

//QFairyLineEdit

class QFairyLineEdit : public QLineEdit
{
    Q_OBJECT

public:
    QFairyLineEdit(const QString &contents, QWidget *parent = 0);

    virtual int valid();
    virtual void searchItem();

public slots:
    void search();
    void appendText(QString);
    void keyboard();

protected:
    virtual void focusOutEvent(QFocusEvent *event);
    virtual void keyPressEvent(QKeyEvent *);
    virtual void mouseDoubleClickEvent(QMouseEvent *event);
    virtual void contextMenuEvent(QContextMenuEvent *event);
};

//QNoEmptyLineEdit

class QNoEmptyLineEdit : public QFairyLineEdit
{
    Q_OBJECT

public:
    QNoEmptyLineEdit(bool show_message, const QString &contents, QWidget *parent = 0);

    virtual int valid();

private:
    bool showMessage;
};

//QDoubleLineEdit

class QDoubleLineEdit : public QFairyLineEdit
{
    Q_OBJECT

public:
    QDoubleLineEdit(const QString &contents, QWidget *parent = 0);

    virtual int valid();
    virtual void searchItem();
};

//QNoEmptyDoubleLineEdit

class QNoEmptyDoubleLineEdit : public QNoEmptyLineEdit
{
    Q_OBJECT

public:
    QNoEmptyDoubleLineEdit(bool show_message, const QString &contents, QWidget *parent = 0);

    virtual int valid();
    virtual void searchItem();

private:
    bool showMessage;
};

//QNoZeroDoubleLineEdit

class QNoZeroDoubleLineEdit : public QDoubleLineEdit
{
    Q_OBJECT

public:
    QNoZeroDoubleLineEdit(bool show_message, const QString &contents, QWidget *parent = 0);

    virtual int valid();

 private:
    bool showMessage;
};

//QDateLineEdit

class QDateLineEdit : public QFairyLineEdit
{
    Q_OBJECT

public:
     QDateLineEdit(bool v, const QString &contents, QWidget *parent = 0);

     virtual int valid();
     virtual void searchItem();

 private:
     bool verify;
};

//QSavePathLineEdit

class QSavePathLineEdit : public QFairyLineEdit
{
    Q_OBJECT

public:
    QSavePathLineEdit (QString title, const QString &contents, QWidget *parent = 0);

    virtual void searchItem();

private:
    QString searchTitle;

};

//QSaveFileLineEdit

class QSaveFileLineEdit : public QFairyLineEdit
{
    Q_OBJECT

public:
    QSaveFileLineEdit (const QString &contents, QWidget *parent = 0);

    virtual void searchItem();
};

#endif //__LINEEDITS_H
