/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QSettings>
#include <QAction>
#include <QGroupBox>
#include <QLabel>
#include <QLineEdit>
#include <QGridLayout>
#include <QSvgWidget>
#include <QSqlDatabase>
#include <QSqlError>
#include <QMessageBox>
#include <QSqlQuery>
#include <QDialog>
#include <QDesktopWidget>

#include "lineedits.h"
#include "global.h"
#include "logindialog.h"

//QLoginDialog

QLoginDialog::QLoginDialog(QWidget *parent)
    : QFairyLoginDialog("Login", parent)
{
    editFontPlus = 3;

    QGroupBox *loginGroupBox = new QGroupBox(tr("&Login"), this);

    QFont font = loginGroupBox->font();
    font.setPointSize(font.pointSize() + editFontPlus);

    QLabel *userLabel = new QLabel(tr("&User"), loginGroupBox);
    userEdit = new QFairyLineEdit(fairy_sql_user, loginGroupBox);
    userLabel->setBuddy(userEdit);

    QLabel *passwordLabel = new QLabel(tr("&Password"), loginGroupBox);
    passwordEdit = new QFairyLineEdit(fairy_sql_passwd, loginGroupBox);
    passwordEdit->setEchoMode(QLineEdit::Password);
    passwordLabel->setBuddy(passwordEdit);

    QLabel *hostLabel = new QLabel(tr("&Host"), loginGroupBox);
    hostEdit = new QFairyLineEdit(fairy_sql_host, loginGroupBox);
    insertLineEditToVerify(hostEdit);
    hostLabel->setBuddy(hostEdit);

    QValidator *portValidator = new QIntValidator(0,10000, loginGroupBox);
    QString portString = QString::number(fairy_sql_port);
    QLabel *portLabel = new QLabel(tr("P&ort"), loginGroupBox);
    portEdit = new QFairyLineEdit(portString, loginGroupBox);
    portEdit->setValidator(portValidator);
    insertLineEditToVerify(portEdit);
    portLabel->setBuddy(portEdit);

    QLabel *socketLabel = new QLabel(tr("&Socket"), loginGroupBox);
    socketEdit = new QFairyLineEdit(fairy_sql_unix_socket, loginGroupBox);
    insertLineEditToVerify(socketEdit);
    socketLabel->setBuddy(socketEdit);

    QGridLayout *loginLayout = new QGridLayout;

    loginLayout->addWidget(userLabel, 0, 0);
    loginLayout->addWidget(userEdit, 0, 1);
    loginLayout->addWidget(passwordLabel, 1, 0);
    loginLayout->addWidget(passwordEdit, 1, 1);
    loginLayout->addWidget(hostLabel, 2, 0);
    loginLayout->addWidget(hostEdit, 2, 1);
    loginLayout->addWidget(portLabel, 3, 0);
    loginLayout->addWidget(portEdit, 3, 1);
    loginLayout->addWidget(socketLabel, 4, 0);
    loginLayout->addWidget(socketEdit, 4, 1);

    loginGroupBox->setLayout(loginLayout);

    QPushButton *okButton = new QPushButton(QIcon(":images/ok.svg"),tr("OK"), this);
    QPushButton *cancelButton = new QPushButton(QIcon(":images/cancel.svg"),tr("Cancel"), this);

    userEdit->setFont(font);
    passwordEdit->setFont(font);
    hostEdit->setFont(font);
    portEdit->setFont(font);
    socketEdit->setFont(font);

    connect(okButton, SIGNAL(clicked()), this, SLOT(verifyFairyInputLines()));
    connect(cancelButton, SIGNAL(clicked()), this, SLOT(reject()));

    QHBoxLayout *buttonLayout = new QHBoxLayout;
    
    buttonLayout->addStretch(1);
    buttonLayout->addWidget(okButton);
    buttonLayout->addWidget(cancelButton);

    QVBoxLayout *svgLayout = new QVBoxLayout;
    QVBoxLayout *editLayout = new QVBoxLayout;

    QHBoxLayout *mainLayout = new QHBoxLayout;

    editLayout->addWidget(loginGroupBox);
    editLayout->addStretch(1);
    editLayout->addLayout(buttonLayout);

    QHBoxLayout *fairyLayout = new QHBoxLayout;
    fairyLayout->setContentsMargins(7,7,7,7);

    QSvgWidget *picture1 = new QSvgWidget(":images/fairy.svg", this);
    picture1->setFixedSize(QSize(120, 180));

    fairyLayout->addWidget(picture1);

    QHBoxLayout *sqlSimbolLayout = new QHBoxLayout;
    QSvgWidget *picture2 = new QSvgWidget(":images/mysql.svg", this);
    picture2->setFixedSize(QSize(67, 67));

    sqlSimbolLayout->addWidget(picture2);

    svgLayout->addLayout(fairyLayout);
    svgLayout->addStretch(0);
    svgLayout->addLayout(sqlSimbolLayout);

    mainLayout->addLayout(svgLayout);
    mainLayout->addLayout(editLayout);
    
    setLayout(mainLayout);

    setWindowIcon(QIcon(":images/fairy.svg"));
    setWindowTitle("fairySQL 0.9.1 - MySQL - Login");

    connect(this, SIGNAL(endVerifyFairyInputLines()), this, SLOT(accept()));

    if (userEdit->text().isEmpty())
        userEdit->setFocus(Qt::OtherFocusReason);
    else if (passwordEdit->text().isEmpty())
        passwordEdit->setFocus(Qt::OtherFocusReason);
}

QString QLoginDialog::getLoginUserName()
{
    return userEdit->text();
}

QString QLoginDialog::getLoginPassword()
{
    return passwordEdit->text();
}

QString QLoginDialog::getLoginHost()
{
    return hostEdit->text();
}

QString QLoginDialog::getLoginPort()
{
    return portEdit->text();
}

QString QLoginDialog::getLoginSocket()
{
    return socketEdit->text();
}
