/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QApplication>
#include <QModelIndex>
#include <QSqlTableModel>
#include <QTreeWidgetItem>
#include <QSqlQuery>
#include <QSqlError>
#include <QSqlRecord>
#include <QMessageBox>
#include <QTextEdit>
#include <QTreeView>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QTableView>
#include <QSplitter>
#include <QSettings>
#include <QToolButton>
#include <QAction>
#include <QLabel>

#include "tree.h"
#include "textedit.h"
#include "tableview.h"
#include "highlighter.h"
#include "formulaeditor.h"
#include "mainwidget.h"
#include "sqlmodel.h"
#include "mainwindow.h"
#include "global.h"

extern MainWindow *mainWindow;

//MainWidget

MainWidget::MainWidget(QWidget *parent) :
    QWidget(parent)
{
    QVBoxLayout *mainLayout = new QVBoxLayout(this);
    mainLayout->setContentsMargins(7,7,7,7);

    horizontalSplitter = new QSplitter(this);
    horizontalSplitter->setOrientation(Qt::Vertical);

    verticalSplitter = new QSplitter(this);
    verticalSplitter->setOrientation(Qt::Horizontal);

    fairySqlModel = new QFairySqlModel(this);

    tree = new QFairyTree(this);
    tree->setColumnCount(3);
    tree->setHeaderLabel("fairySQL");
    tree->hideColumn(1);
    tree->hideColumn(2);

    tablazat = new QFairyTableView(this);
    tablazat->setReadOnly(true);
    tablazat->setModel(fairySqlModel);

    QAction *saveTablesAction = new QAction(QIcon(":images/save.svg"),tr("Save selected tables to file (SQL)"), this);
    connect(saveTablesAction, SIGNAL(triggered()), this, SLOT(saveTables()));
    addAction(saveTablesAction);

    QAction *executeQueryAction = new QAction(QIcon(":images/gear.svg"),tr("Execute query"), this);
    executeQueryAction->setShortcut(QKeySequence(Qt::Key_F9));
    connect(executeQueryAction, SIGNAL(triggered()), this, SLOT(executeQuery()));
    addAction(executeQueryAction);

    QAction *exitAction = new QAction(QIcon(":images/exit.svg"),tr("E&xit"), this);
    exitAction->setShortcut(tr("Alt+X"));
    connect(exitAction, SIGNAL(triggered()), qApp, SLOT(closeAllWindows()));

    QWidget *tablazatWidget = new QWidget(horizontalSplitter);
    QHBoxLayout *tablazatLayout = new QHBoxLayout;

    QVBoxLayout *buttonLayout = new QVBoxLayout;

    QToolButton *saveTablesButton = new QToolButton(this);
    saveTablesButton->setDefaultAction(saveTablesAction);
    saveTablesButton->setFocusPolicy(Qt::NoFocus);
    saveTablesButton->setIconSize(QSize(51,51));

    QToolButton *executeQueryButton = new QToolButton(this);
    executeQueryButton->setDefaultAction(executeQueryAction);
    executeQueryButton->setFocusPolicy(Qt::NoFocus);
    executeQueryButton->setIconSize(QSize(51,51));

    QToolButton *exitButton = new QToolButton(this);
    exitButton->setDefaultAction(exitAction);
    exitButton->setFocusPolicy(Qt::NoFocus);
    exitButton->setIconSize(QSize(51,51));

    buttonLayout->addStretch(1);
    buttonLayout->addWidget(exitButton);

    tablazatLayout->addWidget(tablazat);
    tablazatLayout->addLayout(buttonLayout);
    tablazatWidget->setLayout(tablazatLayout);
    tablazatLayout->setContentsMargins(0,0,0,0);

    tablazat->setAlternatingRowColors(true);
    tablazat->setEditTriggers(QAbstractItemView::AnyKeyPressed | QAbstractItemView::SelectedClicked);

    QHBoxLayout *queryLayout = new QHBoxLayout;
    queryLayout->setContentsMargins(0,0,0,0);
    QWidget *queryWidget = new QWidget(this);
    QVBoxLayout *queryButtonsLayout = new QVBoxLayout;

    queryButtonsLayout->addWidget(saveTablesButton);
    queryButtonsLayout->addWidget(executeQueryButton);
    queryButtonsLayout->addStretch(1);

    status = new QFairyTextEdit(queryWidget);
    QFairyHighlighter *highlighter = new QFairyHighlighter(status->document());

    queryLayout->addWidget(status);
    queryLayout->addLayout(queryButtonsLayout);
    queryWidget->setLayout(queryLayout);

    horizontalSplitter->addWidget(queryWidget);
    horizontalSplitter->addWidget(tablazatWidget);

    verticalSplitter->addWidget(tree);
    verticalSplitter->addWidget(horizontalSplitter);

    mainLayout->addWidget(verticalSplitter);

    QSettings settings;
    horizontalSplitter->restoreState(settings.value("main/horizontalSplitterSizes").toByteArray());
    verticalSplitter->restoreState(settings.value("main/verticalSplitterSizes").toByteArray());

    connect(tree, SIGNAL(itemDoubleClicked(QTreeWidgetItem*,int)), this, SLOT(showSelectedTable(QTreeWidgetItem*,int)));
    connect(tablazat, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(editItem(QModelIndex)));
    connect(fairySqlModel, SIGNAL(selectEditedRow(const QModelIndex&)), this, SLOT(selectEditedRow(const QModelIndex&)));
    connect(tablazat, SIGNAL(setModelQuery(QString,QString)), this, SLOT(setModelQuery(QString,QString)));
    connect(tree, SIGNAL(reloadTreeSignal()), this, SLOT(reloadTree()));
    connect(tree, SIGNAL(selectFromTableSignal(QTreeWidgetItem *)), this, SLOT(selectFromTable(QTreeWidgetItem*)));

    populateTree();

    if (tablazat->model())
    {
        emit modelIsEmpty(false);
    }
    else
    {
        emit modelIsEmpty(true);
    }
    emit readOnlyModel(false);
}

void MainWidget::reloadTree()
{
    tree->clear();
    populateTree();
}

void MainWidget::setModelQuery(const QString &oldQuery, const QString &a_mainId)
{
    fairySqlModel->setQuery(oldQuery, QSqlDatabase::database("fairySQL"));
    if (fairySqlModel->lastError().isValid())
        QMessageBox::critical(this, tr("Error"), fairySqlModel->lastError().text());

    for (int i=0;i<tablazat->model()->rowCount();i++)
    {
        QSqlRecord record = qobject_cast<QFairySqlModel *>(tablazat->model())->record(i);
        QString mainId = record.value(primaryKeyColumn).toString();
        if (mainId == a_mainId)
        {
            QModelIndex modelIndex;
            if (tablazat->model()->rowCount() >1)
                modelIndex = tablazat->model()->index(i, 1);
            else
                modelIndex = tablazat->model()->index(i, 0);
            tablazat->setCurrentIndex(modelIndex);
        }
    }

    tablazat->setFocus();
}

void MainWidget::selectFromTable(QTreeWidgetItem *item)
{
    tablazat->setReadOnly(true);
    QSqlQueryModel *queryModel = new QSqlQueryModel(this);
    if (queryModel)
    {
        QString query = "SELECT * FROM %tn% LIMIT 100";
        query.replace("%tn%", item->parent()->parent()->text(0)+"."+item->text(0));

        status->clear();
        status->setPlainText(query+";");

        queryModel->setQuery(query, QSqlDatabase::database("fairySQL"));
        if (queryModel->lastError().isValid())
        {
            QMessageBox::critical(this, tr("Error"), queryModel->lastError().text());
            return;
        }
        tablazat->setModel(queryModel);
        tablazat->resizeColumnsToContents();
        emit readOnlyModel(true);
    }
    else
        QMessageBox::critical(this, tr("Error"), tr("Error - no queryModel)"));
}

void MainWidget::selectEditedRow(const QModelIndex &index)
{
    tablazat->setFocus();
    tablazat->setCurrentIndex(index);
}

void MainWidget::editItem(QModelIndex modelIndex)
{
    if (tablazat->isReadOnly())
        return;
    int column = modelIndex.column();
    int row = modelIndex.row();

    QSqlQuery modelQuery = qobject_cast<QFairySqlModel *>(tablazat->model())->query();
    QString oldQuery = modelQuery.lastQuery();

    if (primaryKeyColumn == -1)
    {
        QMessageBox::critical(this, tr("Error"), tr("Tables with no primary keys aren't editable"));
        return;
    }

    if (column == primaryKeyColumn)
    {
        QMessageBox::critical(this, tr("Error"), tr("Primary key column isn't editable"));
        return;
    }

    QFieldItem *fieldItem = fieldList.at(column);

    int current = tablazat->currentIndex().row();    
    QSqlRecord record = qobject_cast<QFairySqlModel *>(tablazat->model())->record(current);
    QString mainId = record.value(primaryKeyColumn).toString();

    if ((fieldItem->dataType.toLower() == "text") ||
        (fieldItem->dataType.toLower() == "tinytext") ||
        (fieldItem->dataType.toLower() == "mediumtext") ||
        (fieldItem->dataType.toLower() == "longtext"))
    {
        QSqlQuery getData(QSqlDatabase::database("fairySQL"));
        QString query = "SELECT ";
        query += fieldItem->columnName;
        query += " FROM ";
        query += tree->currentItem()->parent()->parent()->text(0);
        query += ".";
        query += tree->currentItem()->text(0);
        query += " WHERE ";
        query += primaryKeyName;
        query += " = '";
        query += mainId;
        query += "'";

        getData.prepare(query);
        getData.exec();
        if (getData.lastError().isValid())
            QMessageBox::critical(this, tr("Error"), getData.lastError().text());

        QString textField;
        while (getData.next())
        {
            textField = getData.value(0).toString();
        }

        QFairyFormulaEditor *formulaEditor = new QFairyFormulaEditor(textField, tr("Edit text field"), this);
        formulaEditor->exec();
        bool myISave = formulaEditor->textChanged();
        if (myISave)
        {
            textField = formulaEditor->getText();

            QByteArray b;
            b.clear();
            b += textField;

            query += "UNHEX('";
            query += b.toHex();
            query +="')";

            query = "UPDATE %1% SET %2%=UNHEX('%3%') WHERE %4%='%5%'";
            query.replace("%1%", tree->currentItem()->parent()->parent()->text(0)+"."+tree->currentItem()->text(0));
            query.replace("%2%", fieldItem->columnName);
            query.replace("%3%", b.toHex());
            query.replace("%4%", primaryKeyName);
            query.replace("%5%", mainId);

            getData.prepare(query);
            getData.exec();
            if (getData.lastError().isValid())
                QMessageBox::critical(this, tr("Error"), getData.lastError().text());

        }

        delete formulaEditor;
    }
    if ((fieldItem->dataType.toLower() == "blob") ||
        (fieldItem->dataType.toLower() == "tinyblob") ||
        (fieldItem->dataType.toLower() == "mediumblob") ||
        (fieldItem->dataType.toLower() == "longblob"))
    {
        QSqlQuery getData(QSqlDatabase::database("fairySQL"));
        QString query = "SELECT ";
        query += fieldItem->columnName;
        query += " FROM ";
        query += tree->currentItem()->parent()->parent()->text(0);
        query += ".";
        query += tree->currentItem()->text(0);
        query += " WHERE ";
        query += primaryKeyName;
        query += " = '";
        query += mainId;
        query += "'";

        getData.prepare(query);
        getData.exec();
        if (getData.lastError().isValid())
            QMessageBox::critical(this, tr("Error"), getData.lastError().text());

        QByteArray ba1;
        while (getData.next())
        {
            ba1 = getData.value(0).toByteArray();
        }

        QFairyModalDialog *d = new QFairyModalDialog("PhotoViewerDialog", tr("Photo viewer"), this);
        QVBoxLayout *mainLayout = new QVBoxLayout(d);
        QHBoxLayout *photoLayout = new QHBoxLayout;
        QPixmap pic;
        pic.loadFromData(ba1);
        QLabel *picture = new QLabel(d);
        picture->setPixmap(pic);

        photoLayout->setContentsMargins(0,0,0,0);
        photoLayout->addStretch(1);
        photoLayout->addWidget(picture);
        photoLayout->addStretch(1);

        mainLayout->setContentsMargins(0,0,0,0);
        mainLayout->addStretch(1);
        mainLayout->addLayout(photoLayout);
        mainLayout->addStretch(1);

        d->exec();
        delete d;
    }

    fairySqlModel->setQuery(oldQuery, QSqlDatabase::database("fairySQL"));
    if (fairySqlModel->lastError().isValid())
        QMessageBox::critical(this, tr("Error"), fairySqlModel->lastError().text());

    QModelIndex cindex = tablazat->model()->index(row, column);
    tablazat->setCurrentIndex(cindex);
}

void MainWidget::showSelectedTable(QTreeWidgetItem *item, int column)
{
    tablazat->setReadOnly(false);
    QString itemType = item->text(1);
    if ((itemType != "table") && (itemType !="procedure"))
        return;
    QString schemaName = item->parent()->parent()->text(column);
    QString tableName = schemaName + "." + item->text(column);

    if (itemType == "procedure")
    {
        QString query = "SHOW CREATE PROCEDURE " + tableName;

        QSqlQueryModel *tempModel = new QSqlQueryModel(this);
        tempModel->setQuery(query, QSqlDatabase::database("fairySQL"));
        if (tempModel->lastError().isValid())
            QMessageBox::critical(this, tr("Error"), tempModel->lastError().text());

        QSqlRecord record = tempModel->record(0);

        status->clear();
        status->insertPlainText(record.value(2).toString());
        return;
    }

    tablazat->setTableName(tableName);
    tablazat->setModel(fairySqlModel);

    primaryKeyColumn = -1;
    primaryKeyName = "";

    QString query;
    QSqlQuery getData(QSqlDatabase::database("fairySQL"));

    fairySqlModel->clearLists();

    query = "SELECT column_name, ordinal_position "
            "FROM information_schema.columns "
            "WHERE table_schema='%schema_name%' "
            "AND table_name='%table_name%' "
            "AND column_key='PRI'";
    query.replace("%schema_name%", schemaName);
    query.replace("%table_name%", item->text(column));

    getData.prepare(query);
    getData.exec();
    if (getData.lastError().isValid())
        QMessageBox::critical(this, tr("Error"), getData.lastError().text());

    while (getData.next())
    {
        primaryKeyName = getData.value(0).toString();
        primaryKeyColumn = getData.value(1).toInt()-1;
    }

    if (primaryKeyColumn == -1)
    {
        QMessageBox::critical(this, tr("Error"), tr("Table with no primary key. Cannot edit"));
        return;
    }

    fairySqlModel->setFieldValue(primaryKeyColumn, item->text(column), schemaName);

    query = "SELECT column_name, column_default, is_nullable, data_type, character_maximum_length, numeric_precision, numeric_scale, column_type "
            "FROM information_schema.columns "
            "WHERE table_schema='%schema_name%' "
            "AND table_name='%table_name%' ";
    query.replace("%schema_name%", schemaName);
    query.replace("%table_name%", item->text(column));

    getData.prepare(query);
    getData.exec();
    if (getData.lastError().isValid())
        QMessageBox::critical(this, tr("Error"), getData.lastError().text());

    fieldList.clear();
    while (getData.next())
    {
        QFieldItem *fieldItem = new QFieldItem;

        fieldItem->columnName = getData.value(0).toString();
        fieldItem->columnDefault = getData.value(1).toString();
        fieldItem->isNullable = getData.value(2).toBool();
        fieldItem->dataType = getData.value(3).toString();;
        fieldItem->characterMaximumLength = getData.value(4).toInt();
        fieldItem->numericPrecision = getData.value(5).toInt();
        fieldItem->numericScale = getData.value(6).toInt();
        fieldItem->comumnType = getData.value(7).toString();

        fieldList.append(fieldItem);
        fairySqlModel->fieldList.append(fieldItem);
    }

    tablazat->setPrimaryKeyColumn(primaryKeyColumn, fieldList.at(primaryKeyColumn)->columnName);

    for (int i=0;i<fieldList.count();i++)
    {
        if (fieldList.at(i)->dataType.toLower() == "int")
            fairySqlModel->addNumericField(i);
        if (fieldList.at(i)->dataType.toLower() == "varchar")
            fairySqlModel->addCharField(i);
        if (fieldList.at(i)->dataType.toLower() == "date")
            fairySqlModel->addDateField(i);
        if (fieldList.at(i)->dataType.toLower() == "tinyint")
            fairySqlModel->addNumericField(i);
        if (fieldList.at(i)->dataType.toLower() == "smallint")
            fairySqlModel->addNumericField(i);
        if (fieldList.at(i)->dataType.toLower() == "double")
            fairySqlModel->addDoubleField(i);
        if (fieldList.at(i)->dataType.toLower() == "decimal")
            fairySqlModel->addDoubleField(i);
    }

    query = "SELECT ";

    for (int i=0;i<fieldList.count();i++)
    {
        if ((fieldList.at(i)->dataType.toLower() == "text") ||
            (fieldList.at(i)->dataType.toLower() == "tinytext") ||
            (fieldList.at(i)->dataType.toLower() == "mediumtext") ||
            (fieldList.at(i)->dataType.toLower() == "longtext"))
        {
            query += "LEFT(";
            query += fieldList.at(i)->columnName;
            query += ",21)";
        }
        else if((fieldList.at(i)->dataType.toLower() == "blob") ||
                (fieldList.at(i)->dataType.toLower() == "tinyblob") ||
                (fieldList.at(i)->dataType.toLower() == "mediumblob") ||
                (fieldList.at(i)->dataType.toLower() == "longblob"))
        {
            query += "LEFT(HEX(";
            query += fieldList.at(i)->columnName;
            query += "),10)";
        }
        else
        {
            query += fieldList.at(i)->columnName;
            if (i != primaryKeyColumn)
                fairySqlModel->addEditableField(i);
        }
        if (i<fieldList.count()-1)
            query += ",";
    }

    query += " FROM " + tableName;


    fairySqlModel->setQuery(query, QSqlDatabase::database("fairySQL"));
    if (fairySqlModel->lastError().isValid())
        QMessageBox::critical(this, tr("Error"), fairySqlModel->lastError().text());

    for (int i=0;i<fieldList.count();i++)
    {
        fairySqlModel->setHeaderData(i, Qt::Horizontal, fieldList.at(i)->columnName);
    }

    tablazat->resizeColumnsToContents();
    status->clear();
    status->insertPlainText(query);
    status->insertPlainText(";\n");

    if (tablazat->model()->rowCount() == 0)
    {
        emit modelIsEmpty(true);
    }
    else
    {
        emit modelIsEmpty(false);
    }
    emit readOnlyModel(false);
    tablazat->setFocus();
}

void MainWidget::populateTree()
{
    QTreeWidgetItem *server = new QTreeWidgetItem(tree);
    server->setText(0, fairy_sql_host);
    server->setText(1, "server");
    server->setText(2, "root");
    server->setFlags(server->flags() | Qt::ItemIsUserCheckable | Qt::ItemIsTristate);
    server->setCheckState(0, Qt::Unchecked);
    QString query;
    QSqlQuery getData(QSqlDatabase::database("fairySQL"));
    QSqlQueryModel proceduresModel(this);

    query = "SHOW SCHEMAS";

    getData.prepare(query);
    getData.exec();
    if (getData.lastError().isValid())
        QMessageBox::critical(this, tr("Error"), getData.lastError().text());

    QStringList schemas;
    QStringList tables;

    while (getData.next())
    {
        schemas.append(getData.value(0).toString());
    }

    for (int i=0;i<schemas.count();i++)
    {
        QTreeWidgetItem *item = new QTreeWidgetItem(server);
        item->setText(0, schemas.at(i));
        item->setText(1, "schema");
        item->setText(2, "parentchild");
        item->setFlags(item->flags() | Qt::ItemIsUserCheckable | Qt::ItemIsTristate);
        item->setCheckState(0, Qt::Unchecked);

        query = "SHOW TABLES FROM ";
        query += schemas.at(i);

        getData.prepare(query);
        getData.exec();
        if (getData.lastError().isValid())
            QMessageBox::critical(this, tr("Error"), getData.lastError().text());

        QTreeWidgetItem *tablesItem;

        if (getData.size())
        {
            tablesItem = new QTreeWidgetItem(item);
            tablesItem->setText(0, "Tables");
            tablesItem->setText(1, "tables");
            tablesItem->setText(2, "parentchild");
            tablesItem->setFlags(tablesItem->flags() | Qt::ItemIsUserCheckable | Qt::ItemIsTristate);
            tablesItem->setCheckState(0, Qt::Unchecked);
        }

        tables.clear();
        while (getData.next())
        {
            tables.append(getData.value(0).toString());
        }

        for (int j=0;j<tables.count();j++)
        {
            QTreeWidgetItem *table = new QTreeWidgetItem(tablesItem);
            table->setText(0, tables.at(j));
            table->setText(1, "table");
            table->setText(2, "child");
            table->setFlags(item->flags() | Qt::ItemIsUserCheckable);
            table->setCheckState(0, Qt::Unchecked);
            /*
            query = "DESCRIBE " + schemas.at(i) +"."+tables.at(j);

            getData.prepare(query);
            getData.exec();
            if (getData.lastError().isValid())
                QMessageBox::critical(this, tr("Error"), getData.lastError().text());

            while (getData.next())
            {
                QTreeWidgetItem *field = new QTreeWidgetItem(table);
                field->setText(0, getData.value(0).toString());
                field->setText(1, "field");
            }
            */
        }

        query = "SHOW PROCEDURE STATUS WHERE Db='";
        query += schemas.at(i);
        query += "'";

        proceduresModel.setQuery(query, QSqlDatabase::database("fairySQL"));
        if (proceduresModel.lastError().isValid())
            QMessageBox::critical(this, tr("Error"), proceduresModel.lastError().text());

        QTreeWidgetItem *proceduresItem;

        if (proceduresModel.rowCount())
        {
            proceduresItem = new QTreeWidgetItem(item);
            proceduresItem->setText(0, "Stored Procedures");
            proceduresItem->setText(1, "procedures");
            proceduresItem->setText(2, "parentchild");
            proceduresItem->setFlags(tablesItem->flags() | Qt::ItemIsUserCheckable | Qt::ItemIsTristate);
            proceduresItem->setCheckState(0, Qt::Unchecked);
        }

        for (int j=0;j<proceduresModel.rowCount();j++)
        {
            QSqlRecord record = proceduresModel.record(j);
            QTreeWidgetItem *procedure = new QTreeWidgetItem(proceduresItem);
            procedure->setText(0, record.value(1).toString());
            procedure->setText(1, "procedure");
            procedure->setText(2, "child");
            procedure->setFlags(item->flags() | Qt::ItemIsUserCheckable);
            procedure->setCheckState(0, Qt::Unchecked);
        }
    }
}

void MainWidget::executeQuery()
{
    tablazat->setReadOnly(true);
    QSqlQueryModel *queryModel = new QSqlQueryModel(this);
    if (queryModel)
    {
        queryModel->setQuery(status->toPlainText(), QSqlDatabase::database("fairySQL"));
        if (queryModel->lastError().isValid())
        {
            QMessageBox::critical(this, tr("Error"), queryModel->lastError().text());
            return;
        }
        tablazat->setModel(queryModel);
        tablazat->resizeColumnsToContents();
        emit readOnlyModel(true);
    }
    else
        QMessageBox::critical(this, tr("Error"), tr("Error - no queryModel"));

}

MainWidget::~MainWidget()
{
    QSettings settings;
    settings.setValue("main/horizontalSplitterSizes", horizontalSplitter->saveState());
    settings.setValue("main/verticalSplitterSizes", verticalSplitter->saveState());
}
