/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QSqlError>
#include <QMessageBox>
#include <QTreeWidgetItem>
#include <QSqlDatabase>
#include <QSize>
#include <QSettings>
#include <QApplication>
#include <QStatusBar>

#include "formatcolorglobals.h"
#include "sqlglobals.h"
#include "global.h"
#include "mainwidget.h"
#include "mainwindow.h"
#include "textviewer.h"
#include "logindialog.h"

extern MainWindow *mainWindow;

static bool createConnection()
{
    QSqlDatabase db = QSqlDatabase::addDatabase("QMYSQL","fairySQL");
    db.setHostName(fairy_sql_host);
    db.setPort(fairy_sql_port);
    db.open(fairy_sql_user,fairy_sql_passwd);
    if (db.lastError().isValid())
    {
        QMessageBox::critical(0, "Error", db.lastError().text());
        return false;
    }
    return true;
}

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent)
{
    setWindowIcon(QIcon(":images/fairy.svg"));

    QSettings settings;
    bool success = false;

    success = createConnection();

    keywordFormatColorString = settings.value("keywordFormatColor","000080").toString();
    classFormatColorString = settings.value("classFormatColor","FF0000").toString();
    singleLineCommentFormatColorString = settings.value("singleLineCommentFormatColor","C0C0C0").toString();
    quotationFormatColorString = settings.value("quotationFormatColor","008000").toString();
    functionFormatColorString = settings.value("functionFormatColor","0000FF").toString();
    showFieldFormatColorString = settings.value("showFieldFormatColor","808000").toString();
    htmlNumberFormatColorString = settings.value("htmlNumberFormatColor","0000FF").toString();
    dollarNumberFormatColorString = settings.value("dollarNumberFormatColor","800000").toString();
    stringFormatColorString = settings.value("stringFormatColor","008000").toString();

    mainWidget = new MainWidget(this);
    setCentralWidget(mainWidget);

    statusBar()->showMessage(tr("Ready"));

    deskTopColorString = settings.value("deskTopColor","780000").toString();
    fairyCharFieldColorString = settings.value("fairyCharFieldColor","000000").toString();
    fairyNumericFieldColorString = settings.value("fairyNumericFieldColor","007600").toString();
    fairyDateFieldColorString = settings.value("fairyDateFieldColor","AA0000").toString();
    fairyLogicalFieldColorString = settings.value("fairyLogicalFieldColor","550000").toString();

    settings.setValue("deskTopColor", deskTopColorString);
    settings.setValue("fairyCharFieldColor", fairyCharFieldColorString);
    settings.setValue("fairyDateFieldColor", fairyNumericFieldColorString);
    settings.setValue("fairyDateFieldColor", fairyDateFieldColorString );
    settings.setValue("fairyLogicalFieldColor", fairyLogicalFieldColorString);

    settings.setValue("logindialog/username", fairy_sql_user);
    settings.setValue("logindialog/password", fairy_sql_passwd);
    settings.setValue("logindialog/host", fairy_sql_host);
    settings.setValue("logindialog/port", fairy_sql_port);
    settings.setValue("logindialog/socket", fairy_sql_unix_socket);

    settings.setValue("keywordFormatColor", keywordFormatColorString);
    settings.setValue("classFormatColor", classFormatColorString);
    settings.setValue("singleLineCommentFormatColor", singleLineCommentFormatColorString);
    settings.setValue("quotationFormatColor", quotationFormatColorString);
    settings.setValue("functionFormatColor", functionFormatColorString);
    settings.setValue("showFieldFormatColor", showFieldFormatColorString);
    settings.setValue("htmlNumberFormatColor", htmlNumberFormatColorString);
    settings.setValue("dollarNumberFormatColor", dollarNumberFormatColorString);
    settings.setValue("stringFormatColor", stringFormatColorString);

    settings.setValue("logindialog/savepassword", savePassword);
    QSize size = settings.value("main/size", QSize(1300,700)).toSize();
    resize(size);

    mainWindow = this;
    emit setToolButtonIconSize(51);
}

void MainWindow::showListFile(const QString &fileName)
{
    QFairyTextViewer *contgenTextViewer = new QFairyTextViewer(tr("Print file"), this, fileName);
    contgenTextViewer->reloadDocument();
    contgenTextViewer->exec();
    delete contgenTextViewer;
}

MainWindow::~MainWindow()
{
    QSettings settings;
    settings.setValue("main/size", size());
    delete mainWidget;
}
