/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QLabel>
#include <QSettings>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QGridLayout>
#include <QGroupBox>
#include <QCheckBox>
#include <QFileDialog>
#include <QTextEdit>
#include <QSqlError>
#include <QMessageBox>
#include <QSqlRecord>
#include <QStatusBar>
#include <QTreeWidgetItem>
#include <QTextStream>

#include "lineedits.h"
#include "widgets.h"
#include "mainwindow.h"
#include "mainwidget.h"

extern MainWindow *mainWindow;

void MainWidget::saveTables()
{
    QFairyModalDialog *d = new QFairyModalDialog("SaveToSQLDialog", tr("Save to SQL File"), this);
    QVBoxLayout * mainLayout = new QVBoxLayout(d);

    QGroupBox *filtersGroupBox = new QGroupBox(d);
    QGridLayout *filtersLayout = new QGridLayout(filtersGroupBox);

    QCheckBox *checkBox1 = new QCheckBox(tr("Insert DROP SCHEMA"), d);
    QCheckBox *checkBox2 = new QCheckBox(tr("Insert CREATE SCHEMA"), d);
    QCheckBox *checkBox3 = new QCheckBox(tr("Insert DROP TABLE"), d);
    QCheckBox *checkBox4 = new QCheckBox(tr("Insert CREATE TABLE"), d);
    QCheckBox *checkBox5 = new QCheckBox(tr("Insert USE schema-name"), d);

    filtersLayout->addWidget(checkBox1, 0, 0);
    filtersLayout->addWidget(checkBox2, 0, 1);
    filtersLayout->addWidget(checkBox3, 1, 0);
    filtersLayout->addWidget(checkBox4, 1, 1);
    filtersLayout->addWidget(checkBox5, 2, 0, 1, 2/*, Qt::AlignHCenter*/);

    QGroupBox *lineEditsGroupBox = new QGroupBox(d);
    QGridLayout *lineEditsLayout = new QGridLayout(lineEditsGroupBox);

    QLabel *saveFileLabel = new QLabel(tr("File to save"), d);
    QSaveFileLineEdit *saveFileLineEdit = new QSaveFileLineEdit("", d);
    d->insertLineEditToVerify(saveFileLineEdit);
    saveFileLabel->setBuddy(saveFileLineEdit);

    lineEditsLayout->addWidget(saveFileLabel, 0, 0);
    lineEditsLayout->addWidget(saveFileLineEdit, 0, 1);

    QPushButton *okButton = new QPushButton(QIcon(":images/save.svg"),tr("Save"), this);
    QPushButton *cancelButton = new QPushButton(QIcon(":images/cancel.svg"),tr("Cancel"), this);

    connect(okButton, SIGNAL(clicked()), d, SLOT(verifyFairyInputLines()));
    connect(cancelButton, SIGNAL(clicked()), d, SLOT(reject()));

    QHBoxLayout *buttonLayout = new QHBoxLayout;

    buttonLayout->addStretch(1);
    buttonLayout->addWidget(okButton);
    buttonLayout->addWidget(cancelButton);

    mainLayout->addWidget(filtersGroupBox);
    mainLayout->addWidget(lineEditsGroupBox);
    mainLayout->addStretch(1);
    mainLayout->addLayout(buttonLayout);

    QSettings settings;
    bool dropSchema = settings.value("SaveToSQLDialog/dropSchema", false).toBool();
    bool dropTable = settings.value("SaveToSQLDialog/dropTable", false).toBool();
    bool createSchema = settings.value("SaveToSQLDialog/createSchema", false).toBool();
    bool createTable = settings.value("SaveToSQLDialog/createTable", false).toBool();
    bool useSchema = settings.value("SaveToSQLDialog/useSchema", false).toBool();
    QString saveFileName = settings.value("SaveToSQLDialog/saveFileName", "").toString();

    saveFileLineEdit->setText(saveFileName);

    checkBox1->setChecked(dropSchema);
    checkBox2->setChecked(createSchema);
    checkBox3->setChecked(dropTable);
    checkBox4->setChecked(createTable);
    checkBox5->setChecked(useSchema);

    d->exec();
    if (d->result() == QDialog::Accepted)
    {
        dropSchema = checkBox1->isChecked();
        createSchema = checkBox2->isChecked();
        dropTable = checkBox3->isChecked();
        createTable = checkBox4->isChecked();
        useSchema = checkBox5->isChecked();

        saveFileName = saveFileLineEdit->text();

        settings.setValue("SaveToSQLDialog/dropSchema", dropSchema);
        settings.setValue("SaveToSQLDialog/dropTable", dropTable);
        settings.setValue("SaveToSQLDialog/createSchema", createSchema);
        settings.setValue("SaveToSQLDialog/createTable", createTable);
        settings.setValue("SaveToSQLDialog/useSchema", useSchema);
        settings.setValue("SaveToSQLDialog/saveFileName", saveFileName);
    }
    else
        return;

    delete d;

    QFile textFile;
    QTextStream *text;

    if (!saveFileName.isEmpty())
    {
        if (!saveFileName.endsWith(".sql"))
            saveFileName += ".sql";
        textFile.setFileName(saveFileName);

        if (!textFile.open(QIODevice::WriteOnly | QIODevice::Text))
            return;

        text = new QTextStream(&textFile);
        text->setCodec("UTF-8");
    }
    else
        return;

    QString query;
    QSqlQueryModel getData(this);
    QString schemaName, tableName;
    QTreeWidgetItemIterator it(tree);
    while (*it)
    {
        if (((*it)->checkState(0) == Qt::Checked) || ((*it)->checkState(0) == Qt::PartiallyChecked))
        {
            if ((*it)->text(1) == "schema")
            {
                schemaName = (*it)->text(0);
                if (!schemaName.isEmpty())
                {                    
                    query = "DROP SCHEMA IF EXISTS `" + schemaName + "`;\n\n";
                    if (dropSchema)
                        *text << query;
                    query = "CREATE SCHEMA `" + schemaName + "`;\n\n";
                    if (createSchema)
                        *text << query;
                    query = "USE `" + schemaName + "`;\n\n";
                    if (useSchema)
                        *text << query;
                }
            }
            if ((*it)->text(1) == "table")
            {
                tableName = (*it)->text(0);

                if (!tableName.isEmpty())
                {
                    QString tableRowsString = "";

                    query = "SELECT table_rows FROM information_schema.tables WHERE table_schema='%ts%' AND table_name='%tn%'";
                    query.replace("%ts%", schemaName).replace("%tn%", tableName);

                    getData.setQuery(query, QSqlDatabase::database("fairySQL"));
                    if (getData.lastError().isValid())
                        QMessageBox::critical(this, tr("Error"), getData.lastError().text());

                    QSqlRecord record = getData.record(0);

                    tableRowsString = record.value(0).toString();

                    QString message;

                    message = tr("Saving ")+ tableRowsString + tr(" rows from ") +schemaName + "." + tableName;
                    mainWindow->statusBar()->showMessage(message);

                    query = "SHOW CREATE TABLE ";
                    query += schemaName + "." + tableName;

                    getData.setQuery(query, QSqlDatabase::database("fairySQL"));
                    if (getData.lastError().isValid())
                        QMessageBox::critical(this, tr("Error"), getData.lastError().text());

                    record = getData.record(0);

                    query = "DROP TABLE IF EXISTS `"+ (*it)->text(0) + "`;\n\n";
                    if (dropTable)
                        *text << query;
                    if (createTable)
                    {
                        *text << record.value(1).toString().replace("CREATE TABLE ", "CREATE TABLE IF NOT EXISTS ");
                        *text << ";";
                    }
                    *text << "\n\n";

                    QList<QFieldItem *> fieldList;

                    fieldList.clear();

                    query = "SELECT "
                            "column_name, "
                            "column_default, "
                            "is_nullable, "
                            "data_type, "
                            "character_maximum_length, "
                            "numeric_precision, "
                            "numeric_scale, "
                            "column_type "
                            "FROM information_schema.columns "
                            "WHERE table_schema='%schema_name%' "
                            "AND table_name='%table_name%' ";
                    query.replace("%schema_name%", schemaName);
                    query.replace("%table_name%", tableName);

                    getData.setQuery(query, QSqlDatabase::database("fairySQL"));
                    if (getData.lastError().isValid())
                        QMessageBox::critical(this, tr("Error"), getData.lastError().text());

                    for (int i=0;i<getData.rowCount();i++)
                    {
                        record = getData.record(i);
                        QFieldItem *fieldItem = new QFieldItem;
                        fieldItem->columnName = record.value(0).toString();
                        fieldItem->columnDefault = record.value(1).toString();
                        fieldItem->isNullable = record.value(2).toString();
                        fieldItem->dataType = record.value(3).toString();;
                        fieldItem->characterMaximumLength = record.value(4).toInt();
                        fieldItem->numericPrecision = record.value(5).toInt();
                        fieldItem->numericScale = record.value(6).toInt();
                        fieldItem->comumnType = record.value(7).toString();

                        fieldList.append(fieldItem);
                    }

                    query = "SELECT ";
                    for (int i=0;i<fieldList.count();i++)
                    {
                        if(
                                (fieldList.at(i)->dataType.toLower() == "blob") ||
                                (fieldList.at(i)->dataType.toLower() == "tinyblob") ||
                                (fieldList.at(i)->dataType.toLower() == "mediumblob") ||
                                (fieldList.at(i)->dataType.toLower() == "longblog")
                          )
                        {
                            query += "HEX(";
                            query += fieldList.at(i)->columnName;
                            query += ")";
                        }
                        else if(
                                (fieldList.at(i)->dataType.toLower() == "text") ||
                                (fieldList.at(i)->dataType.toLower() == "tinytext") ||
                                (fieldList.at(i)->dataType.toLower() == "mediumtext") ||
                                (fieldList.at(i)->dataType.toLower() == "longtext") ||
                                (fieldList.at(i)->dataType.toLower() == "char") ||
                                (fieldList.at(i)->dataType.toLower() == "varchar")
                               )
                        {
                            if (fieldList.at(i)->isNullable == "YES")
                            {
                                query += " IF("+fieldList.at(i)->columnName+" IS NULL, 'NULL', QUOTE("+fieldList.at(i)->columnName+")) ";
                            }
                            else
                            {
                                query += "QUOTE(";
                                query += fieldList.at(i)->columnName;
                                query += ")";
                            }
                        }
                        else
                        {
                            if (fieldList.at(i)->isNullable == "YES")
                            {
                                query += " IF("+fieldList.at(i)->columnName+" IS NULL, 'NULL', "+fieldList.at(i)->columnName+") ";
                            }
                            else
                                query += fieldList.at(i)->columnName;
                        }
                        if (i < fieldList.count()-1)
                            query += ",";
                    }
                    query += " FROM " + schemaName + "." + tableName;

                    getData.setQuery(query, QSqlDatabase::database("fairySQL"));
                    if (getData.lastError().isValid())
                        QMessageBox::critical(this, tr("Error"), getData.lastError().text());

                    if (getData.rowCount())
                    {
                        *text << "INSERT INTO `"+tableName+"` (";
                        for (int i=0;i<fieldList.count();i++)
                        {
                            *text << "`" << fieldList.at(i)->columnName << "`";
                            if (i<fieldList.count()-1)
                                *text << ",";
                        }
                        *text << ") VALUES \n";
                    }

                    double rowLength = 0;
                    for (int i=0;i<getData.rowCount();i++)
                    {
                        *text << "\t(";
                        record = getData.record(i);
                        for (int j=0;j<fieldList.count();j++)
                        {
                            if(
                                    (fieldList.at(j)->dataType.toLower() == "blob") ||
                                    (fieldList.at(j)->dataType.toLower() == "tinyblob") ||
                                    (fieldList.at(j)->dataType.toLower() == "mediumblob") ||
                                    (fieldList.at(j)->dataType.toLower() == "longblog")
                              )
                            {
                                if ((record.value(j).toString() == "NULL") && (fieldList.at(j)->isNullable == "YES"))
                                    *text << "NULL";
                                else if (!record.value(j).toString().isEmpty())
                                {
                                    *text << "_binary 0x";
                                    *text <<  record.value(j).toString();
                                }
                                else
                                    *text << "''";
                            }
                            else
                            {
                                if ((record.value(j).toString() == "NULL") && (fieldList.at(j)->isNullable == "YES"))
                                    *text << "NULL";
                                else
                                {
                                    if ((fieldList.at(j)->dataType.toLower() == "text") ||
                                        (fieldList.at(j)->dataType.toLower() == "tinytext") ||
                                        (fieldList.at(j)->dataType.toLower() == "mediumtext") ||
                                        (fieldList.at(j)->dataType.toLower() == "longtext") ||
                                        (fieldList.at(j)->dataType.toLower() == "char") ||
                                         (fieldList.at(j)->dataType.toLower() == "varchar")
                                       )
                                    {
                                        *text << record.value(j).toString().replace('\n', "\\n");
                                    }
                                    else
                                        *text << "'" << record.value(j).toString() << "'";
                                }
                            }
                            rowLength += record.value(j).toString().length();
                            if (j<fieldList.count()-1)
                                *text << ",";
                        }
                        *text << ")";
                        if (rowLength > 900000)
                        {
                            *text << ";\nINSERT INTO `"+tableName+"` (";
                            for (int i=0;i<fieldList.count();i++)
                            {
                                *text << "`" << fieldList.at(i)->columnName << "`";
                                if (i<fieldList.count()-1)
                                    *text << ",";
                            }
                            *text << ") VALUES \n";
                            rowLength = 0;
                        }
                        else
                            if (i< getData.rowCount()-1)
                                *text << ",\n";
                    }
                    *text << ";\n\n";

                }
            }
            if ((*it)->text(1) == "procedure")
            {
                tableName = (*it)->text(0);

                if (!tableName.isEmpty())
                {
                    query = "SHOW CREATE PROCEDURE " + schemaName + "." + tableName;

                    getData.setQuery(query, QSqlDatabase::database("fairySQL"));
                    if (getData.lastError().isValid())
                        QMessageBox::critical(this, tr("Error"), getData.lastError().text());

                    QSqlRecord record = getData.record(0);

                    query = "DROP PROCEDURE IF EXISTS `"+ (*it)->text(0) + "`;\n\n";
                    query += "DELIMITER //\n\n";
                    *text << query;
                    *text << record.value(2).toString();
                    *text << "//\nDELIMITER ;\n\n";
                }
            }
        }
        ++it;
    }
    if (text)
        delete text;
    mainWindow->statusBar()->clearMessage();
    mainWindow->statusBar()->showMessage("Saving to file "+saveFileName+" done.", 3000);
}
