/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QDate>
#include <QLocale>
#include <QColor>
#include <QFont>
#include <QApplication>
#include <QSqlQuery>
#include <QSqlRecord>
#include <QSqlError>

#include "sqlmodel.h"
#include "sqlglobals.h"

//QFairySqlModel

QFairySqlModel::QFairySqlModel(QObject *parent, QSqlDatabase db)
    : QSqlQueryModel(parent)
{
    clearLists();
}

void QFairySqlModel::clearLists()
{
    charFields.clear();
    numericFields.clear();
    dateFields.clear();
    logicalFields.clear();
    textFields.clear();
    blobFields.clear();
    doubleFields.clear();
    editableFields.clear();
    fieldList.clear();
}

Qt::ItemFlags QFairySqlModel::flags(
        const QModelIndex &index) const
{
    Qt::ItemFlags flags = QSqlQueryModel::flags(index);
    for (int i=0;i<editableFields.count();i++)
    {
        if (index.column() == editableFields.at(i))
            flags |= Qt::ItemIsEditable;
    }
    return flags;
}


bool QFairySqlModel::setData(const QModelIndex &index, const QVariant &value, int role )
{
    //if (index.column() < 1 || index.column() > 2)
    //    return false;

    //QModelIndex primaryKeyIndex = QSqlQueryModel::index(index.row(), primaryIndexColumn);
    //int id = data(primaryKeyIndex).toInt();
    QSqlQuery modelQuery = this->query();
    QString oldQuery = modelQuery.lastQuery();

    QSqlRecord record = this->record(index.row());
    QString mainId = record.value(primaryIndexColumn).toString();

    //clear();

    bool ok = false;
    for (int i=0;i<editableFields.count();i++)
    {
        if (index.column() == editableFields.at(i))
        {
            ok = saveFieldValue(value.toString(), index.column(), mainId);
        }
    }

    this->setQuery(oldQuery, QSqlDatabase::database("fairySQL"));
    emit selectEditedRow(index);
    return ok;
}

void QFairySqlModel::setFieldValue(int a_primaryIndexColumn,
                                   const QString &a_tableName,
                                   const QString &a_schemaName)
{
    primaryIndexColumn = a_primaryIndexColumn;
    tableName = a_tableName;
    schemaName = a_schemaName;
}

bool QFairySqlModel::saveFieldValue(const QString &fieldValue, int editedColumn, const QString &a_mainId)
{
    QFieldItem *fieldItem = fieldList.at(editedColumn);
    QFieldItem *primaryItem = fieldList.at(primaryIndexColumn);

    QString query;
    QSqlQuery updateQuery(QSqlDatabase::database("fairySQL"));;

    query = ("UPDATE %1%.%2% SET %3% = '%4%' WHERE %5%='%6%'");
    query.replace("%1%", schemaName);
    query.replace("%2%", tableName);
    query.replace("%3%", fieldItem->columnName);
    query.replace("%4%", fieldValue);
    query.replace("%5%", primaryItem->columnName);
    query.replace("%6%", a_mainId);

    updateQuery.prepare(query);
    return updateQuery.exec();
}


void QFairySqlModel::addCharField(int i)
{
    charFields.append(i);
}

void QFairySqlModel::addNumericField(int i)
{
    numericFields.append(i);
}

void QFairySqlModel::addDateField(int i)
{
    dateFields.append(i);
}

void QFairySqlModel::addLogicalField(int i)
{
    logicalFields.append(i);
}

void QFairySqlModel::addTextField(int i)
{
    textFields.append(i);
}

void QFairySqlModel::addBlobField(int i)
{
    blobFields.append(i);
}

void QFairySqlModel::addDoubleField(int i)
{
    doubleFields.append(i);
}

void QFairySqlModel::addEditableField(int i)
{
    editableFields.append(i);
}

QVariant QFairySqlModel::data(const QModelIndex &index, int role) const
{

    QVariant value = QSqlQueryModel::data(index, role);
    int i;

    QLocale clocale(QLocale::C);
    quint8 r,g,b;
    QByteArray tempByteArray;

    for (i=0;i<numericFields.count();i++)
    {
        if (index.column() == numericFields.at(i))
        {
            switch (role)
            {
            case Qt::TextAlignmentRole:
                value = QVariant(Qt::AlignRight | Qt::AlignVCenter);
                break;
            case Qt::TextColorRole:
                tempByteArray.append(fairyNumericFieldColorString);
                tempByteArray = QByteArray::fromHex(tempByteArray);
                r = tempByteArray.at(0);
                g = tempByteArray.at(1);
                b = tempByteArray.at(2);
                QColor tempColor(r,g,b);
                value = qVariantFromValue(tempColor);
                break;
            }
        }
    }

    for (i=0;i<doubleFields.count();i++)
    {
        if (index.column() == doubleFields.at(i))
        {
            switch (role)
            {
            case Qt::DisplayRole:
                value = clocale.toString(value.toDouble(), 'f', 2);
                break;
            case Qt::TextAlignmentRole:
                value = QVariant(Qt::AlignRight | Qt::AlignVCenter);
                break;
            case Qt::TextColorRole:
                tempByteArray.append(fairyNumericFieldColorString);
                tempByteArray = QByteArray::fromHex(tempByteArray);
                r = tempByteArray.at(0);
                g = tempByteArray.at(1);
                b = tempByteArray.at(2);
                QColor tempColor(r,g,b);
                value = qVariantFromValue(tempColor);
                break;
            }
        }
    }

    for (i=0;i<dateFields.count();i++)
    {
        if (index.column() == dateFields.at(i))
        {
            switch (role)
            {
            case Qt::DisplayRole:
                value = value.toDate().toString("dd.MM.yyyy");
                break;
            case Qt::TextAlignmentRole:
                value = QVariant(Qt::AlignRight | Qt::AlignVCenter);
                break;
            case Qt::TextColorRole:
                tempByteArray.append(fairyDateFieldColorString);
                tempByteArray = QByteArray::fromHex(tempByteArray);
                r = tempByteArray.at(0);
                g = tempByteArray.at(1);
                b = tempByteArray.at(2);
                QColor tempColor(r,g,b);
                value = qVariantFromValue(tempColor);
                break;
            }
        }
    }

    for (i=0;i<logicalFields.count();i++)
    {
        if (index.column() == logicalFields.at(i))
        {
            switch (role)
            {
            case Qt::TextAlignmentRole:
                value = QVariant(Qt::AlignLeft | Qt::AlignVCenter);
                break;
            case Qt::TextColorRole:
                tempByteArray.append(fairyLogicalFieldColorString);
                tempByteArray = QByteArray::fromHex(tempByteArray);
                r = tempByteArray.at(0);
                g = tempByteArray.at(1);
                b = tempByteArray.at(2);
                QColor tempColor(r,g,b);
                value = qVariantFromValue(tempColor);
                break;
            }
        }
    }

    return value;
}
