/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QFileDialog>
#include <QMessageBox>
#include <QSqlQuery>
#include <QSqlDatabase>
#include <QSqlError>
#include <QSqlRecord>
#include <QContextMenuEvent>
#include <QMenu>
#include <QAction>
#include <QTextStream>

#include "sqlmodel.h"
#include "sqlglobals.h"
#include "mainwindow.h"
#include "tableview.h"

extern MainWindow *mainWindow;

//QFairyTableView

QFairyTableView::QFairyTableView(QWidget *parent) :
    QTableView(parent)
{
    readOnly = false;
    modelIsEmpty = false;

    connect(this, SIGNAL(insertSignal()), this, SLOT(insertRecord()));
    connect(this, SIGNAL(deleteSignal()), this, SLOT(deleteRecord()));
}

bool QFairyTableView::isReadOnly()
{
    return readOnly;
}

void QFairyTableView::setReadOnly(bool a_readOnly)
{
    readOnly = a_readOnly;
}

void QFairyTableView::exportAsCSV()
{
    QFile textFile;
    QTextStream *text;
    QSqlQuery getData(QSqlDatabase::database("fairySQL"));
    QString query;
    QString foundedValue;

    QString fileName = QFileDialog::getSaveFileName(this);
    if (!fileName.isEmpty())
    {
        if (!fileName.endsWith(".csv"))
            fileName += ".csv";
        textFile.setFileName(fileName);

        if (!textFile.open(QIODevice::WriteOnly | QIODevice::Text))
            return;

        text = new QTextStream(&textFile);
        text->setCodec("UTF-8");


        QModelIndex modelIndex;
        double proba;
        bool ok;

        for (int i=0;i<this->model()->rowCount();i++)
        {
            for (int j=0;j<this->model()->columnCount();j++)
            {
                modelIndex = this->model()->index(i,j);

                foundedValue = this->model()->data(modelIndex).toString();

                QRegExp re1("\\d*");  // a digit (\d), zero or more times (*)
                QRegExp re2("^[-,0-9.]+$");  // a digit (\d), zero or more times (*)

                if (re1.exactMatch(foundedValue))
                {
                    *text << '"' << this->model()->data(modelIndex).toString() << '"';
                }
                else
                    if (re2.exactMatch(foundedValue))
                    {
                        proba = this->model()->data(modelIndex).toString().toDouble(&ok);

                        if (ok)
                        {
                            *text << '"' << proba << '"';
                        }
                        else
                            *text << '"' << this->model()->data(modelIndex).toString() << '"';
                    }
                    else
                        *text << '"' << this->model()->data(modelIndex).toString() << '"';


                /*
                    *text << this->model()->data(modelIndex).toString();
                    proba = this->model()->data(modelIndex).toString().toDouble(&ok);

                    if (ok)
                        {
                            *text << '"' << proba << '"';
                        }
                    else
                        {
                            *text << '"' << this->model()->data(modelIndex).toString() << '"';
                        }

                    */
                //*text << this->model()->data(modelIndex).toString();
                if (j<this->model()->columnCount()-1)
                    *text << ",";
            }

            *text << "\n";
        }

        delete text;

    }
}

void QFairyTableView::listTheTable()
{
    QString outputFileName;
    QFile textFile;
    QTextStream *text;
    QString foundedValue;

    outputFileName = "table.html";

    textFile.setFileName(outputFileName);

    if (!textFile.open(QIODevice::WriteOnly | QIODevice::Text))
        return;

    text = new QTextStream(&textFile);
    text->setCodec("UTF-8");


    QModelIndex modelIndex;
    //double proba;
    //bool ok;

    *text << "<html>" << endl;

    *text << "<meta http-equiv='Content-Type' content='text/html; charset=utf-8'>" << endl;
    *text << "<body>" << endl;
    *text << "<table width='100%' border='1' cellpadding='1' cellspacing='0'>" << endl;
    *text << "<thead>" << endl;
    //*text << "<td align='center' valign='top'><b><font face='arial' size=3>Nr</font></b></td>" << endl;

    for (int i=0;i<this->model()->rowCount();i++)
    {
        *text << "<tr>" << endl;
        for (int j=0;j<this->model()->columnCount();j++)
        {
            modelIndex = this->model()->index(i,j);

            foundedValue = this->model()->data(modelIndex).toString();

            QRegExp re1("\\d*");  // a digit (\d), zero or more times (*)
            QRegExp re2("^[-,0-9.]+$");  // a digit (\d), zero or more times (*)
            if (re1.exactMatch(foundedValue))
            {
                *text << "<td align='left' valign='top'>" ;
            }
            else
                if (re2.exactMatch(foundedValue))
                    *text << "<td align='right' valign='top'>" ;
                else
                    *text << "<td align='left' valign='top'>" ;



            *text << this->model()->data(modelIndex).toString();
            if (j<this->model()->columnCount()-1)
                *text << "</td>";

        }

        *text << "</tr>\n";
    }

    *text << "</table></body></html>" << endl;

    delete text;

    mainWindow->showListFile(outputFileName);
}

void QFairyTableView::contextMenuEvent(QContextMenuEvent *event)
{
    QAction *exportAct = new QAction(QIcon(":images/formule.svg"), tr("Export table contents to csv"), this);
    connect(exportAct, SIGNAL(triggered()), this, SLOT(exportAsCSV()));

    QAction *listTableAct = new QAction(QIcon(":images/printer.svg"), tr("List the table"), this);
    connect(listTableAct, SIGNAL(triggered()), this, SLOT(listTheTable()));

    QAction *rowsToContentsAct = new QAction(QIcon(":images/table.svg"), tr("Resize rows to contents"), this);
    connect(rowsToContentsAct, SIGNAL(triggered()), this, SLOT(resizeRowsToContents()));

    QAction *insertRecordAct = 0;
    QAction *deleteRecordAct = 0;

    if (!readOnly)
    {

        insertRecordAct = new QAction(QIcon(":images/menure.svg"), tr("Insert record"), this);
        connect(insertRecordAct, SIGNAL(triggered()), this, SLOT(insertRecord()));

        deleteRecordAct = new QAction(QIcon(":images/menun.svg"), tr("Delete selected records"), this);
        connect(deleteRecordAct, SIGNAL(triggered()), this, SLOT(deleteRecord()));

        if (modelIsEmpty)
        {
            deleteRecordAct->setDisabled(true);
        }
        else
        {
            deleteRecordAct->setDisabled(false);
        }
    }

    QMenu *menu = new QMenu(this);
    menu->addAction(exportAct);
    menu->addAction(listTableAct);
    menu->addAction(rowsToContentsAct);
    if (!readOnly)
    {
        menu->addSeparator();
        menu->addAction(insertRecordAct);
        menu->addAction(deleteRecordAct);
    }

    menu->exec(event->globalPos());
    delete menu;
}

void QFairyTableView::insertRecord()
{
    QSqlQuery modelQuery = qobject_cast<QFairySqlModel *>(this->model())->query();
    QString oldQuery = modelQuery.lastQuery();

    QString query = "INSERT INTO ";
    query += tableName;
    query += " (";
    QSqlQuery insertQuery(QSqlDatabase::database("fairySQL"));
    for (int i=0;i < qobject_cast<QFairySqlModel *>(this->model())->fieldList.count();i++)
    {
        QFieldItem * fieldItem = qobject_cast<QFairySqlModel *>(this->model())->fieldList.at(i);
        query += fieldItem->columnName;
        if (i<qobject_cast<QFairySqlModel *>(this->model())->fieldList.count()-1)
            query += ",";
    }
    query += ") VALUES (";
    for (int i=0;i < qobject_cast<QFairySqlModel *>(this->model())->fieldList.count();i++)
    {
        QFieldItem * fieldItem = qobject_cast<QFairySqlModel *>(this->model())->fieldList.at(i);
        query += "'";
        query += fieldItem->columnDefault;
        query += "'";
        if (i<qobject_cast<QFairySqlModel *>(this->model())->fieldList.count()-1)
            query += ",";
    }
    query += ")";

    insertQuery.prepare(query);
    insertQuery.exec();
    if (insertQuery.lastError().isValid())
        QMessageBox::critical(this, tr("Error"), insertQuery.lastError().text());

    query = "SELECT last_insert_id()";
    insertQuery.prepare(query);
    insertQuery.exec();
    if (insertQuery.lastError().isValid())
        QMessageBox::critical(this, tr("Error"), insertQuery.lastError().text());

    QString mainId;

    while (insertQuery.next())
    {
        mainId = insertQuery.value(0).toString();
    }

    emit setModelQuery(oldQuery, mainId);
}

void QFairyTableView::setPrimaryKeyColumn(int column, const QString &columnName)
{
    primaryKeyColumn = column;
    primaryKeyColumnName = columnName;
}

void QFairyTableView::deleteRecord()
{
    QSqlQuery modelQuery = qobject_cast<QFairySqlModel *>(this->model())->query();
    QString oldQuery = modelQuery.lastQuery();

    QString query;

    QModelIndexList currentSelection = this->selectionModel()->selectedRows();

    if (currentSelection.count() == 0)
    {
        QMessageBox::critical(this, tr("Error"), tr("Select at least a row"));
        return;
    }

    QString selectedIds = "(";

    for (int i=0;i<currentSelection.count();i++)
    {
        QSqlRecord record = qobject_cast<QFairySqlModel *>(this->model())->record(currentSelection.at(i).row());
        selectedIds += record.value(primaryKeyColumn).toString();
        if (i<currentSelection.count()-1)
            selectedIds += ",";
    }
    selectedIds += ")";

    query = "DELETE FROM ";
    query += tableName;
    query += " WHERE ";
    query += primaryKeyColumnName;
    query += " IN ";
    query += selectedIds;

    QSqlQueryModel *proceedModel = new QSqlQueryModel(this);
    QMessageBox::StandardButton reply;
    reply = QMessageBox::question(0, tr("Delete"),tr("<h1>You are to delete selected rows.</h1>"
                                                       "<h1 align='right'><font color='red'>Are you sure?</font></h1>"),
                                  QMessageBox::Yes | QMessageBox::No );
    if (reply == QMessageBox::Yes)
    {
        proceedModel->setQuery(query, QSqlDatabase::database("fairySQL"));
        if (proceedModel->lastError().isValid())
            QMessageBox::critical(this, tr("Eroare"), proceedModel->lastError().text());

        emit setModelQuery(oldQuery, "");
    }
}

void QFairyTableView::setTableName(const QString &a_tableName)
{
    tableName = a_tableName;
}

void QFairyTableView::keyPressEvent(QKeyEvent *event)
{
    QModelIndex index = this->currentIndex();

    switch (event->key())
    {
    case Qt::Key_Enter:
    case Qt::Key_Return:
        emit enterSignal(index);
        if (model()->rowCount() != 0)
            emit noemptyEnterSignal();
        else
            QTableView::keyPressEvent(event);
        break;
    case Qt::Key_F9:
        emit f9Signal();
        break;
    case Qt::Key_F3:
        if (model()->rowCount() != 0)
            emit f3Signal();
        break;
    case Qt::Key_Insert:
        if (event->modifiers() & Qt::ControlModifier)
        {
            emit controlInsertSignal();
            if (model()->rowCount() != 0)
                emit noemptyControlInsertSignal();
        }
        else
        {
            emit insertSignal();
            if (model()->rowCount() != 0)
                emit noemptyInsertSignal();
        }
        break;
    //case Qt::Key_F8:
        //dateDialog();
        //break;
    case Qt::Key_Delete:
        if (model()->rowCount() != 0)
            emit deleteSignal();
        break;
    case Qt::Key_Backspace:
        emit backspaceSignal();
        break;
    case Qt::Key_L:
        emit lcharSignal();
        break;
    case Qt::Key_V:
        emit vcharSignal();
        break;
    default:
        QTableView::keyPressEvent(event);
    }
}
