/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QFontDialog>
#include <QContextMenuEvent>
#include <QMenu>
#include <QAction>
#include <QIcon>
#include <QApplication>
#include <QFileDialog>
#include <QFileInfo>
#include <QTextStream>
#include <QSettings>
#include <QPrinter>

#include "finddialog.h"
#include "textedit.h"

//QFairyTextEdit

QFairyTextEdit::QFairyTextEdit(QWidget *parent)
    : QTextEdit(parent)
{
    QSettings settings;
    //bool texteditreadonly = settings.value("texteditreadonly",true).toBool();
    //setReadOnly(texteditreadonly);
    //settings.setValue("texteditreadonly", texteditreadonly);

    textEditFont = new QFont();

#ifdef UNIX
    textEditFont->setFamily(settings.value("QFairyTextEdit/family","Monospace").toString());
#else
    textEditFont->setFamily(settings.value("QFairyTextEdit/family","Consolas").toString());
#endif
    textEditFont->setPointSize(settings.value("QFairyTextEdit/pointsize",11).toInt());
    textEditFont->setWeight(settings.value("QFairyTextEdit/weight",QFont::Normal).toInt());

    findDialog = new QFairyFindDialog("QFairyFindDialog", tr("Find"), this);

    connect(findDialog, SIGNAL(findFirst(const QString&)), this, SLOT(findNext(const QString&)));
}

void QFairyTextEdit::setContents(const QString &fileName)
{
    QFileInfo fi(fileName);
    srcUrl = QUrl::fromLocalFile(fi.absoluteFilePath());
    QFile file(fileName);
    if (file.open(QIODevice::ReadOnly))
    {
        QTextStream textStream(&file);
        textStream.setCodec("UTF-8");
        QString data(textStream.readAll());
        if (fileName.endsWith(".html") || fileName.endsWith(".htm") || fileName.endsWith(".lsh"))
            setHtml(data);
        else
            setPlainText(data);
    }
}

void QFairyTextEdit::find()
{
    findDialog->show();
    this->document()->setModified(false);
}

void QFairyTextEdit::findNext(const QString &findString)
{
    textCursor = this->document()->find(findString, textCursor);
    this->setTextCursor(textCursor);
}

void QFairyTextEdit::setFont()
{
    bool ifModified = this->document()->isModified();
    selectAll();
    *textEditFont = QFontDialog::getFont(0,this->currentFont());
    this->setCurrentFont(*textEditFont);
    this->document()->setModified(ifModified);
}

void QFairyTextEdit::saveToFile()
{
    QSettings settings;
    QString fileName;
    QString formulaDirectory = settings.value("QFairyTextEdit/directory", ".").toString();
    fileName = QFileDialog::getSaveFileName(this, tr("Save to"), formulaDirectory, "*");
    if (fileName.isEmpty())
    {
        return;
    }

    QFileInfo fileInfo(fileName);
    settings.setValue("QFairyTextEdit/directory", fileInfo.absolutePath());

    QFile file(fileName);
    if (file.open(QIODevice::WriteOnly))
    {
        QTextStream textStream(&file);
        textStream.setCodec("UTF-8");
        QApplication::setOverrideCursor(Qt::WaitCursor);
        textStream << this->document()->toPlainText();
        QApplication::restoreOverrideCursor();
    }

}

void QFairyTextEdit::loadFile()
{
    QSettings settings;
    QString fileName;
    QString formulaDirectory = settings.value("QFairyTextEdit/directory", ".").toString();
    fileName = QFileDialog::getOpenFileName(this, tr("Open File"), formulaDirectory, "*");
    if (fileName.isEmpty())
    {
        return;
    }

    clear();

    QFileInfo fileInfo(fileName);
    settings.setValue("QFairyTextEdit/directory", fileInfo.absolutePath());

    QFile file(fileName);
    if (file.open(QIODevice::ReadOnly))
    {
        QTextStream textStream(&file);
        textStream.setCodec("UTF-8");
        QString data(textStream.readAll());
        setPlainText(data);
    }
}

void QFairyTextEdit::contextMenuEvent(QContextMenuEvent *event)
{
    QAction *saveAct = new QAction(QIcon(":images/save.svg"), tr("Save to text file"), this);
    connect(saveAct, SIGNAL(triggered()), this, SLOT(saveToFile()));
    QAction *loadAct = new QAction(QIcon(":images/load.svg"), tr("Load a text file"), this);
    connect(loadAct, SIGNAL(triggered()), this, SLOT(loadFile()));
    QAction *searchAct = new QAction(QIcon(":images/search.svg"), tr("Find - ^F"), this);
    connect(searchAct, SIGNAL(triggered()), this, SLOT(find()));
    QAction *fontAct = new QAction(QIcon(":images/font.svg"), tr("Set editor font"), this);
    connect(fontAct, SIGNAL(triggered()), this, SLOT(setFont()));

    QMenu *menu = this->createStandardContextMenu();
    menu->addAction(saveAct);
    menu->addAction(loadAct);
    menu->addAction(searchAct);
    menu->addAction(fontAct);
    menu->exec(event->globalPos());
    delete menu;
}

QVariant QFairyTextEdit::loadResource(int type, const QUrl &name)
{
    if (type == QTextDocument::ImageResource)
    {
        QFile file(srcUrl.resolved(name).toLocalFile());
        if (file.open(QIODevice::ReadOnly))
            return file.readAll();
    }
    return QTextEdit::loadResource(type, name);
}

void QFairyTextEdit::print(QPrinter *printer) const
{
    QTextEdit::print(printer);
}

QFairyTextEdit::~QFairyTextEdit()
{
    QSettings settings;
    settings.setValue("QFairyTextEdit/family", textEditFont->family());
    settings.setValue("QFairyTextEdit/pointsize", textEditFont->pointSize());
    settings.setValue("QFairyTextEdit/weight", textEditFont->weight());
}
