/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QSettings>
#include <QWebView>
#include <QWebFrame>
#include <QAction>
#include <QVBoxLayout>
#include <QFontDialog>
#include <QFileInfo>
#include <QPrinter>
#include <QPrintDialog>
#include <QPrintPreviewDialog>
#include <QPrintPreviewWidget>
#include <QFileDialog>
#include <QInputDialog>
#include <QMessageBox>
#include <QKeyEvent>
#include <QTextDocumentWriter>
#include <QTextDocument>

#include "global.h"
#include "textedit.h"
#include "widgets.h"
#include "textviewer.h"

//QFairyTextViewer

QFairyTextViewer::QFairyTextViewer(const QString &title, QWidget *parent, const QString &fileName)
    : QFairyModalDialog(fileName, title, parent)
{
    listFileName = fileName;

#ifdef UNIX
    fairyFont = new QFont("Monospace", 11, QFont::Bold);
#else
    fairyFont = new QFont("Lucida Console", 11, QFont::Bold);
#endif

    readSettings();

    if ((fileName.endsWith(".html")) || (fileName.endsWith(".htm")))
    {
        webView = new QWebView(this);
        isHtml = true;
    }
    else
    {
        if (fileName.endsWith(".lsh"))
        {
            textEdit = new QFairyTextEdit(this);
            isHtml = false;
        }
        if (fileName.endsWith(".txt"))
        {
            textEdit = new QFairyTextEdit(this);
            isHtml = false;
        }
        if (fileName.endsWith(".lst"))
        {
            textEdit = new QFairyTextEdit(this);
            isHtml = false;
        }
    }

    QAction *printAction = new QAction(QIcon(":images/printer.svg"),tr("Print - F9"), this);
    printAction->setShortcut(Qt::Key_F9);
    connect(printAction, SIGNAL(triggered()), this, SLOT(filePrint()));
    if (isHtml)
        webView->addAction(printAction);
    else
        textEdit->addAction(printAction);

    QAction *printPreviewAction = new QAction(QIcon(":images/print-preview.svg"),tr("Print Preview"), this);
    connect(printPreviewAction, SIGNAL(triggered()), this, SLOT(filePrintPreview()));
    if (isHtml)
        webView->addAction(printPreviewAction);
    else
        textEdit->addAction(printPreviewAction);

    QAction *exportPdfAction = new QAction(QIcon(":images/pdf.svg"),tr("Export in fisier PDF - F11"), this);
    exportPdfAction->setShortcut(Qt::Key_F11);
    connect(exportPdfAction, SIGNAL(triggered()), this, SLOT(filePrintPdf()));
    if (isHtml)
        webView->addAction(exportPdfAction);
    else
        textEdit->addAction(exportPdfAction);

    QAction *exportOdfAction = new QAction(QIcon(":images/ooo-gulls.svg"),tr("Export in fisier ODF"), this);
    connect(exportOdfAction, SIGNAL(triggered()), this, SLOT(filePrintOdf()));
    if (isHtml)
        webView->addAction(exportOdfAction);
    else
        textEdit->addAction(exportOdfAction);

    QAction *fontAction = new QAction(QIcon(":images/font.svg"),tr("Schimbare font pentru document"), this);
    fontAction->setShortcut(Qt::Key_F);
    connect(fontAction, SIGNAL(triggered()), this, SLOT(setListFont()));
    if (!isHtml)
        textEdit->addAction(fontAction);

    QAction *dotPrintAction = new QAction(QIcon(":images/dotmatrix.svg"),tr("Print to dot matrix printer - F12"), this);
    dotPrintAction->setShortcut(Qt::Key_F12);
    connect(dotPrintAction, SIGNAL(triggered()), this, SLOT(fileDotPrint()));
    if (!isHtml)
        textEdit->addAction(dotPrintAction);

    QAction *quitAction = new QAction(QIcon(":images/exit.svg"),tr("Inchide - Esc"), this);
    connect(quitAction, SIGNAL(triggered()), this, SLOT(close()));
    if (isHtml)
        webView->addAction(quitAction);
    else
        textEdit->addAction(quitAction);

    QAction *internetAction = new QAction(QIcon(":images/internet.svg"),tr("Navigate the web"), this);
    connect(internetAction, SIGNAL(triggered()), this, SLOT(internet()));
    if (isHtml)
        webView->addAction(internetAction);

    printButton = new QFairyToolButton(this);
    printButton->setDefaultAction(printAction);
    printPreviewButton = new QFairyToolButton(this);
    printPreviewButton->setDefaultAction(printPreviewAction);
    exportPdfButton = new QFairyToolButton(this);
    exportPdfButton->setDefaultAction(exportPdfAction);
    exportOdfButton = new QFairyToolButton(this);
    exportOdfButton->setDefaultAction(exportOdfAction);
    if (!isHtml)
    {
        fontButton = new QFairyToolButton(this);
        fontButton->setDefaultAction(fontAction);
        dotPrintButton = new QFairyToolButton(this);
        dotPrintButton->setDefaultAction(dotPrintAction);
    }

    printButton->setFocusPolicy(Qt::NoFocus);
    printPreviewButton->setFocusPolicy(Qt::NoFocus);
    exportPdfButton->setFocusPolicy(Qt::NoFocus);
    if (!isHtml)
    {
        fontButton->setFocusPolicy(Qt::NoFocus);
        dotPrintButton->setFocusPolicy(Qt::NoFocus);
    }

    QVBoxLayout *buttonLayout = new QVBoxLayout;

    buttonLayout->addWidget(printButton);
    buttonLayout->addWidget(printPreviewButton);
    buttonLayout->addWidget(exportPdfButton);
    buttonLayout->addWidget(exportOdfButton);
    if (!isHtml)
    {
        buttonLayout->addWidget(fontButton);
        buttonLayout->addWidget(dotPrintButton);
    }
    buttonLayout->addStretch(1);

    setWindowTitle(title);

    QHBoxLayout *mainLayout = new QHBoxLayout(this);
    if (isHtml)
        mainLayout->addWidget(webView);
    else
        mainLayout->addWidget(textEdit);
    mainLayout->addLayout(buttonLayout);
    setLayout(mainLayout);

    connect(parent, SIGNAL(setToolButtonIconSize(int)), this, SLOT(setToolButtonIconSize(int)));
    setToolButtonIconSize(51);
}

void QFairyTextViewer::setListFont()
{
    if (!isHtml)
    {
        textEdit->selectAll();
        *fairyFont = QFontDialog::getFont(0,textEdit->currentFont());
        textEdit->setCurrentFont(*fairyFont);
    }
}

void QFairyTextViewer::writeSettings()
{
    QSettings settings;

#ifndef QT_NO_PRINTER
    QString centrFileName;
    QFileInfo f(listFileName);
    centrFileName = f.baseName();
    settings.setValue(centrFileName+"/orientation", ori);
    settings.setValue(centrFileName+"/papersize", psize);

    settings.setValue(centrFileName+"/left", left);
    settings.setValue(centrFileName+"/right", right);
    settings.setValue(centrFileName+"/top", top);
    settings.setValue(centrFileName+"/bottom", bottom);

    settings.setValue(centrFileName+"/family", fairyFont->family());
    settings.setValue(centrFileName+"/pointsize", fairyFont->pointSize());
    settings.setValue(centrFileName+"/weight", fairyFont->weight());
#endif
}

void QFairyTextViewer::readSettings()
{
    QSettings settings;
#ifndef QT_NO_PRINTER
    QString centrFileName;
    QFileInfo f(listFileName);
    centrFileName = f.baseName();

    ori = settings.value(centrFileName+"/orientation", 0).toInt();
    psize = settings.value(centrFileName+"/papersize", 0).toInt();

    left = settings.value(centrFileName+"/left", 10.).toDouble();
    top = settings.value(centrFileName+"/top", 10.).toDouble();
    right = settings.value(centrFileName+"/right", 10.).toDouble();
    bottom = settings.value(centrFileName+"/bottom", 10.).toDouble();

#ifdef UNIX
    fairyFont->setFamily(settings.value(centrFileName+"/family","Monospace").toString());
#else
    fairyFont->setFamily(settings.value(centrFileName+"/family","Lucida console").toString());
#endif
    fairyFont->setPointSize(settings.value(centrFileName+"/pointsize",10).toInt());
    fairyFont->setWeight(settings.value(centrFileName+"/weight",QFont::Normal).toInt());

#endif
}


void QFairyTextViewer::getFairyPrinterStatus(QPrinter *printer)
{
    printer->getPageMargins(&left,&top,&right,&bottom,QPrinter::Millimeter);
    ori = printer->orientation();
    psize = printer->paperSize();
}

void QFairyTextViewer::setFairyPrinterStatus(QPrinter *printer)
{
    switch (ori)
    {
    case 0:
        printer->setOrientation(QPrinter::Portrait);
        break;
    case 1:
        printer->setOrientation(QPrinter::Landscape);
        break;
    }

    switch (psize)
    {
    case 0:
        printer->setPaperSize(QPrinter::A4);
        break;
    case 1:
        printer->setPaperSize(QPrinter::B5);
        break;
    case 2:
        printer->setPaperSize(QPrinter::Letter);
        break;
    case 3:
        printer->setPaperSize(QPrinter::Legal);
        break;
    case 4:
        printer->setPaperSize(QPrinter::Executive);
        break;
    case 5:
        printer->setPaperSize(QPrinter::A0);
        break;
    case 6:
        printer->setPaperSize(QPrinter::A1);
        break;
    case 7:
        printer->setPaperSize(QPrinter::A2);
        break;
    case 8:
        printer->setPaperSize(QPrinter::A3);
        break;
    case 9:
        printer->setPaperSize(QPrinter::A5);
        break;
    }

    printer->setPageMargins(left,top,right,bottom,QPrinter::Millimeter);
}


void QFairyTextViewer::filePrint()
{
#ifndef QT_NO_PRINTER
    QPrinter printer(QPrinter::HighResolution);
    setFairyPrinterStatus(&printer);
    QPrintDialog *dlg = new QPrintDialog(&printer, this);
    dlg->setWindowTitle(tr("Print Document"));
    if (dlg->exec() == QDialog::Accepted)
    {
        if (!isHtml)
            textEdit->print(&printer);
        else
            webView->page()->mainFrame()->print(&printer);
        getFairyPrinterStatus(&printer);
    }
    delete dlg;
#endif
}

void QFairyTextViewer::filePrintPreview()
{
#ifndef QT_NO_PRINTER
    QSettings settings;
    QPrinter printer(QPrinter::HighResolution);
    setFairyPrinterStatus(&printer);
    QPrintPreviewDialog preview(&printer, this);
    QSize size=settings.value("printpreview/size",QSize(1024,600)).toSize();
    preview.resize(size);
    connect(&preview, SIGNAL(paintRequested(QPrinter *)), this, SLOT(printPreview(QPrinter *)));
    preview.exec();
    getFairyPrinterStatus(&printer);
    size = preview.size();
    settings.setValue("printpreview/size",size);
#endif
}

void QFairyTextViewer::printPreview(QPrinter *printer)
{
#ifndef QT_NO_PRINTER
    if (!isHtml)
        textEdit->print(printer);
    else
        webView->page()->mainFrame()->print(printer);
#endif
}


void QFairyTextViewer::filePrintPdf()
{
#ifndef QT_NO_PRINTER

    QString fileName = QFileDialog::getSaveFileName(this, tr("Export PDF"),
                                                    QString(), "*.pdf");
    if (!fileName.isEmpty())
    {
        if (QFileInfo(fileName).suffix().isEmpty())
            fileName.append(".pdf");
        QPrinter printer(QPrinter::HighResolution);
        setFairyPrinterStatus(&printer);
        printer.setOutputFormat(QPrinter::PdfFormat);
        printer.setOutputFileName(fileName);
        if (!isHtml)
            textEdit->document()->print(&printer);
        else
            webView->page()->mainFrame()->print(&printer);
    }
#endif
}

void QFairyTextViewer::filePrintOdf()
{
    QString fileName = QFileDialog::getSaveFileName(this, tr("Export ODF"),
                                                    QString(), "*.odt");
    if (!fileName.isEmpty())
    {
        if (QFileInfo(fileName).suffix().isEmpty())
            fileName.append(".odt");

        QTextDocumentWriter *tdw = new QTextDocumentWriter(fileName,"odf");
        if (isHtml)
        {
            QString t = webView->page()->mainFrame()->toHtml();
            QTextDocument *td = new QTextDocument(this);
            td->setHtml(t);
            tdw->write(td);

        }
        else
        {
            tdw->write(textEdit->document());
        }
        delete tdw;
    }
}


void QFairyTextViewer::internet()
{
    bool ok;
    QString searchString;
    searchString = QInputDialog::getText(this, tr("URL"),
                                         tr("URL:"), QLineEdit::Normal,
                                         "", &ok);
    if (ok && !searchString.isEmpty())
    {
        webView->page()->mainFrame()->load(QUrl(searchString));
    }
}

void QFairyTextViewer::fileDotPrint()
{
    char printFileName[1024];
    FILE* fileToPrint;
    char buffer[1024];

    strcpy(printFileName, qPrintable(listFileName));

    fileToPrint = fopen(printFileName,"r");
    if( fileToPrint == NULL )
    {
        QMessageBox::critical(this, tr("Eroare"), tr("Nu pot deschide fisierul pentru listare"));
        return;
    }
#if defined( UNIX )
    FILE* f;
    f = popen("lpr -l","w");
    if (f == NULL)
    {
        QMessageBox::critical(this, tr("Eroare"), tr("Nu pot deschide imprimanta pentru listare"));
        return;
    }
    while (!feof(fileToPrint))
    {
        fgets(buffer, 1024, fileToPrint);
        if( !feof(fileToPrint) )
            fprintf(f,"%s", buffer);
    }
    pclose(f);
#else
    FILE* f;
    f = fopen("lpt1","wt");
    if (f == NULL)
    {
        QMessageBox::critical(this, tr("Eroare"), tr("Nu pot deschide imprimanta pentru listare"));
        return;
    }
    while (!feof(fileToPrint))
    {
        fgets(buffer, 1024, fileToPrint);
        if( !feof(fileToPrint) )
            fprintf(f,"%s", buffer);
    }
    fclose(f);
#endif
    fclose(fileToPrint);
}

void QFairyTextViewer::setToolButtonIconSize(int i)
{
    QSize size = QSize(i,i);
    printButton->setIconSize(size);
    printPreviewButton->setIconSize(size);
    exportPdfButton->setIconSize(size);
    exportOdfButton->setIconSize(size);
    if (!isHtml)
    {
        fontButton->setIconSize(size);
        dotPrintButton->setIconSize(size);
    }
}


void QFairyTextViewer::reloadDocument()
{
    if ((listFileName.endsWith(".html")) || (listFileName.endsWith(".htm")))
    {
        QFileInfo fi(listFileName);
        QUrl srcUrl = QUrl::fromLocalFile(fi.absoluteFilePath());
        webView->load(srcUrl);
        webView->show();
        isHtml = true;
    }
    else
    {
        if (listFileName.endsWith(".lsh"))
        {
            textEdit->setContents(listFileName);
            textEdit->selectAll();
            textEdit->setCurrentFont(*fairyFont);
            textEdit->moveCursor(QTextCursor::Start);
            isHtml = false;
        }
        if (listFileName.endsWith(".txt"))
        {
            textEdit->setContents(listFileName);
            textEdit->selectAll();
            textEdit->setCurrentFont(*fairyFont);
            textEdit->moveCursor(QTextCursor::Start);
            isHtml = false;
        }
        if (listFileName.endsWith(".lst"))
        {
            textEdit->setLineWrapMode(QTextEdit::NoWrap);
            textEdit->setCurrentFont(*fairyFont);
            textEdit->setContents(listFileName);
            isHtml = false;
        }
    }
}
