#include <QAction>
#include <QMenu>
#include <QContextMenuEvent>
#include <QTreeWidgetItem>

#include "tree.h"

QFairyTree::QFairyTree(QWidget *parent)
    : QTreeWidget(parent)
{

}

void QFairyTree::contextMenuEvent(QContextMenuEvent *event)
{
    QAction *selectAct;

    QAction *reloadAct = new QAction(QIcon(":images/refresh.svg"), tr("Reload tree"), this);
    connect(reloadAct, SIGNAL(triggered()), this, SLOT(reloadTree()));

    QAction *expandAllAct = new QAction(QIcon(":images/load.svg"), tr("Expand all"), this);
    connect(expandAllAct, SIGNAL(triggered()), this, SLOT(expandAll()));

    QAction *collapseAllAct = new QAction(QIcon(":images/close.svg"), tr("Collapse all"), this);
    connect(collapseAllAct, SIGNAL(triggered()), this, SLOT(collapseAll()));

    if (this->currentItem()->text(1) == "table")
    {
        selectAct = new QAction(QIcon(":images/table.svg"), tr("Select * from table limit 100"), this);
        connect(selectAct, SIGNAL(triggered()), this, SLOT(selectFromTable()));
    }

    QMenu *menu = new QMenu(this);
    menu->addAction(reloadAct);
    menu->addAction(expandAllAct);
    menu->addAction(collapseAllAct);
    if (this->currentItem()->text(1) == "table")
    {
        menu->addSeparator();
        menu->addAction(selectAct);
    }
    menu->exec(event->globalPos());
    delete menu;
}

void QFairyTree::reloadTree()
{
    emit reloadTreeSignal();
}

void QFairyTree::selectFromTable()
{
    QTreeWidgetItem *item = this->currentItem();
    emit selectFromTableSignal(item);
}
