/****************************************************************************
**
** Copyright (C) 2014 Hevele Hegyi Istvan.
**
** This file is part of the fairySQL project.
**
** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** $QT_BEGIN_LICENSE:BSD$
** You may use this file under the terms of the BSD license as follows:
**
** "Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are
** met:
**   * Redistributions of source code must retain the above copyright
**     notice, this list of conditions and the following disclaimer.
**   * Redistributions in binary form must reproduce the above copyright
**     notice, this list of conditions and the following disclaimer in
**     the documentation and/or other materials provided with the
**     distribution.
**   * Neither the name of Digia Plc and its Subsidiary(-ies) nor the names
**     of its contributors may be used to endorse or promote products derived
**     from this software without specific prior written permission.
**
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
** OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
**
** $QT_END_LICENSE$
**
****************************************************************************/

#include <QHBoxLayout>
#include <QKeyEvent>
#include <QAction>
#include <QMenu>
#include <QSqlQuery>
#include <QFileDialog>
#include <QDate>
#include <QCalendarWidget>
#include <QSqlError>
#include <QMessageBox>
#include <QSettings>
#include <QDesktopWidget>

#include "global.h"
#include "widgets.h"
#include "lineedits.h"
#include "mainwindow.h"

//extern MainWindow *mainWindow;

// QFairyWaitDialog

QFairyWaitDialog::QFairyWaitDialog(QWidget *parent)
    : QDialog(parent)
{
    QHBoxLayout *mainLayout = new QHBoxLayout(this);

    QToolButton *picture = new QToolButton(this);
    picture->setIcon(QIcon(":images/wait.svg"));
    picture->setIconSize(pushButtonIconSize);

    mainLayout->addWidget(picture);
}

// QFairyModalDialog

QFairyModalDialog::QFairyModalDialog(const QString &a_objectName, const QString &title, QWidget *parent)
    : QDialog(parent)
{
    setWindowTitle(title);
    setObjectName(a_objectName);

    QDesktopWidget desk;

    QPoint initialPos;

    QSize initialSize;

    initialSize.setWidth(desk.width()*3/4);
    initialSize.setHeight(desk.height()*3/4);

    initialPos.setX(desk.width()/2-initialSize.width()/2);
    initialPos.setY(desk.height()/2-initialSize.height()/2);

    //initialSize.setWidth(mainWindow->width()/3);
    //initialSize.setHeight(mainWindow->height()/4);

    //initialPos.setX(mainWindow->width()/2-initialSize.width()/2);
    //initialPos.setY(mainWindow->height()/2-initialSize.height()/2);

    QSettings settings;
    QSize size = settings.value(objectName()+"/Size", initialSize).toSize();
    QPoint pos = settings.value(objectName()+"/Position", initialPos).toPoint();

    resize(size);
    move(pos);

    lineEditsToVerify.clear();
    connect(this, SIGNAL(endVerifyFairyInputLines()), this, SLOT(accept()));
}

void QFairyModalDialog::verifyFairyInputLines()
{
    int v = 1;

    QListIterator<QFairyLineEdit *> i(lineEditsToVerify);
    while (i.hasNext())
    {
        int tv;
        tv =i.next()->valid();
        if (tv == 0)
        {
            v = 0;
            break;
        }
    }

    if (v == 1)
        emit endVerifyFairyInputLines();
}

void QFairyModalDialog::insertLineEditToVerify(QFairyLineEdit* fairyLineEdit)
{
    lineEditsToVerify.append(fairyLineEdit);
}


QFairyModalDialog::~QFairyModalDialog()
{
    lineEditsToVerify.clear();
    QSettings settings;
    settings.setValue(objectName()+"/Size", size());
    settings.setValue(objectName()+"/Position", pos());
}

// QFairyLoginDialog

QFairyLoginDialog::QFairyLoginDialog(const QString &title, QWidget *parent)
    : QDialog(parent)
{
    setWindowTitle(title);
    QDesktopWidget desk;

    QPoint initialPos;

    QSize initialSize;

    initialSize.setWidth(desk.width()/3);
    initialSize.setHeight(desk.height()/4);

    initialPos.setX(desk.width()/2-initialSize.width()/2);
    initialPos.setY(desk.height()/2-initialSize.height()/2);

    QSettings settings;
    QSize size = settings.value(objectName()+"/Size", initialSize).toSize();
    QPoint pos = settings.value(objectName()+"/Position", initialPos).toPoint();

    resize(size);
    move(pos);

    //initialSize.setWidth(mainWindow->width()/3);
    //initialSize.setHeight(mainWindow->height()/4);

    //initialPos.setX(mainWindow->width()/2-initialSize.width()/2);
    //initialPos.setY(mainWindow->height()/2-initialSize.height()/2);
}

void QFairyLoginDialog::verifyFairyInputLines()
{
    int v = 1;

    QListIterator<QFairyLineEdit *> i(lineEditsToVerify);
    while (i.hasNext())
    {
        int tv;
        tv =i.next()->valid();
        if (tv == 0)
        {
            v = 0;
            break;
        }
    }

    if (v == 1)
        emit endVerifyFairyInputLines();
}

void QFairyLoginDialog::insertLineEditToVerify(QFairyLineEdit* fairyLineEdit)
{
    lineEditsToVerify.append(fairyLineEdit);
}

QFairyLoginDialog::~QFairyLoginDialog()
{
    lineEditsToVerify.clear();
    QSettings settings;
    settings.setValue(objectName()+"/Size", size());
    settings.setValue(objectName()+"/Position", pos());
}

//QFairyToolButton

QFairyToolButton::QFairyToolButton(QWidget *parent)
    : QToolButton(parent)
{
    setIconSize(toolButtonSize);
}

QSize QFairyToolButton::sizeHint() const
{
    QSize size = QToolButton::sizeHint();
    size.rheight() += 5;
    size.rwidth() = qMax(size.width(), size.height());
    return size;
}

void QFairyToolButton::setIconSize1(int i)
{
    QSize size = QSize(i,i);
    setIconSize(size);
}

//QFairyComboBox

QFairyComboBox::QFairyComboBox(QWidget *parent)
    : QComboBox(parent)
{
}

void QFairyComboBox::keyPressEvent(QKeyEvent *event)
{
    switch (event->key())
    {
    case Qt::Key_Enter:
    case Qt::Key_Return:
        emit enterSignal();
        break;
    case Qt::Key_Insert:
        emit insertSignal();
        break;
    case Qt::Key_Delete:
        if (count() != 0)
            emit deleteSignal();
        break;
    case Qt::Key_F3:
        if (count() != 0)
            emit f3Signal();
        break;
    case Qt::Key_Backspace:
        emit backspaceSignal();
        break;
    default:
        QComboBox::keyPressEvent(event);
    }
}

//QFairyPushButton

QFairyPushButton::QFairyPushButton(const QIcon &icon, const QString &text , QWidget *parent)
    : QPushButton(icon, text, parent)
{
}

QSize QFairyPushButton::sizeHint() const
{
    return iconSize();
}
